/*
 *  Copyright (C) 2012  Jakub Cizek, Marian Vlcek 
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *  If you publish any work which uses this program and/or data
 *  produced by this program, please cite following paper: 
 * 
 *  J. Cizek. M. Vlcek, I. Prochazka, Digital spectrometer
 *  for coincidence measurement of Doppler broadening of positron
 *  annihilation radiation, Nuclear Instruments and Methods
 *  in Physics Research, Section A 623, 982-994 (2010)
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define nlength 3
#define maxch 10000
#define maxhist_small 4000
#define maxhist_small2 500
#define maxhist 10000
#define maxhist_2D 300
#define maxhist_px 10000
#define maxhist_py 10000
#define max_sess 1000
#define wav_add 30000 //co se pricte k waveformu aby byl vzdy kladny



/*zjisti jestli soubor existuje*/
/*kdyz ano 1, kdyz ne 0*/
int fexist(char fname[])
{
	FILE *f;
	if((f=fopen(fname,"r"))==NULL){
				printf("error: file %s does not exist.\n", fname);
				return(0);
			}
	fclose(f);
	return(1);
}

void lfit(double *a,double *b,double *chi2, short int sp[],int i_l,int i_r)  
{
	double sum0,sumx1,sumx2,sumyx,sumy;
	double delta;
	int i;

	sumx1=sumx2=sumy=sumyx=sum0=0.0;
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+i;
		sumx2=sumx2+pow((double)i,2);
		sumy=sumy+sp[i];
		sumyx=sumyx+sp[i]*i;
	}
		delta=sum0*sumx2-pow(sumx1,2);
		*a=(sum0*sumyx-sumy*sumx1)/delta;
		*b=(sumy*sumx2-sumyx*sumx1)/delta;
	*chi2=0.0;
	for(i=i_l;i<=i_r;i++)
	{
		*chi2=*chi2+pow(*a*i+*b-sp[i],2);
	}
	
}

void ilfit(double *a,double *b,double *chi2, int sp[],int i_l,int i_r)  
{
	double sum0,sumx1,sumx2,sumyx,sumy;
	double delta;
	int i;

	sumx1=sumx2=sumy=sumyx=sum0=0.0;
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+i;
		sumx2=sumx2+pow((double)i,2);
		sumy=sumy+sp[i];
		sumyx=sumyx+sp[i]*i;
	}
		delta=sum0*sumx2-pow(sumx1,2);
		*a=(sum0*sumyx-sumy*sumx1)/delta;
		*b=(sumy*sumx2-sumyx*sumx1)/delta;
	*chi2=0.0;
	for(i=i_l;i<=i_r;i++)
	{
		*chi2=*chi2+pow(*a*i+*b-sp[i],2);
	}
	
}

void trans(double a[nlength][nlength], double b[nlength][nlength])
{
	int r, s;
	for (r = 0; r < nlength; r++) {
		for (s = 0; s < nlength; s++) {
			b[s][r] = a[r][s];
		}
	}
}

void printMatrix(double a[nlength][nlength])
{
	int r, s;
	for (r = 0; r < nlength; r++) {
		for (s = 0; s < nlength; s++) {
			printf("%.2f ",a[r][s]);
		}
		printf("\n");
	}
}

// solves system Ax=b, A is upper triangular matrix
void solve_upper(double a[nlength][nlength], double x[nlength], double b[nlength])
{
	int i, j;
	for (i = nlength - 1; i >= 0; i--) {
		x[i] = b[i];
		for (j = nlength - 1; j > i; j--) {
			x[i] = x[i] - a[i][j] * x[j];
		}
		x[i] = x[i] / a[i][i];
	}
}

// solves system Ax=b, A is lower triangular matrix
void solve_lower(double a[nlength][nlength], double x[nlength], double b[nlength])
{
	int i, j;
	for (i = 0; i < nlength; i++) {
		x[i] = b[i];
		for (j = 0; j < i; j++) {
			x[i] = x[i] - a[i][j] * x[j];
		}
		x[i] = x[i] / a[i][i];
	}
}

// Cholesky matrix decomposition A=LL'
void cholesky(double a[nlength][nlength], double l[nlength][nlength])
{
	int i, j, k;
	for (i = 0; i < nlength; i++) {
		l[i][i] = a[i][i];
		for (k = 0; k < i; k++) {
			l[i][i] -= l[i][k] * l[i][k];
		}
		l[i][i] = sqrt(l[i][i]);
		for (j = i + 1; j < nlength; j++) {
			l[j][i] = a[j][i];
			for (k = 0; k < i; k++) {
				l[j][i] -= l[j][k] * l[i][k];
			}
			l[j][i] = l[j][i] / l[i][i];
		}
	}
}

void sqfit2(double *a2, double *a1, double *a0, double *chi2, int sp[], int i_l, int i_r)
{
	int i;
	double sum0,sumx1,sumx2,sumx3,sumx4,sumy,sumxy,sumx2y;
	double as2,as1,as0;
	double delta; 
	double chi2_sum,mod;
	
	double J[nlength][nlength];

	double L[nlength][nlength];
	double LT[nlength][nlength];	
	
	double a[nlength];
	double prava[nlength];
	double y[nlength];	

	sum0=0;
	sumx1=0;
	sumx2=0;
	sumx3=0;
	sumx4=0;
	sumy=0;
	sumxy=0;
	sumx2y=0;
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+(double)i;
		sumx2=sumx2+pow((double)i,2);
		sumx3=sumx3+pow((double)i,3);
		sumx4=sumx4+pow((double)i,4);
		sumy=sumy+sp[i];
		sumxy=sumxy+(double)i*(double)sp[i];
		sumx2y=sumx2y+pow((double)i,2)*(double)sp[i];
	}

	J[0][0]=sum0;
	J[0][1]=J[1][0]=sumx1;
	J[1][1]=J[0][2]=J[2][0]=sumx2;
	J[2][2]=sumx4;
	J[1][2]=J[2][1]=sumx3;
	prava[0]=sumy;
	prava[1]=sumxy;
	prava[2]=sumx2y;
//	printMatrix(J);
		cholesky(J, L);
//	printMatrix(L);		
		trans(L, LT);
//	printMatrix(LT);		
		solve_lower(L, y, prava);
		solve_upper(LT, a, y);
	as0=a[0];
	as1=a[1];
	as2=a[2];
	chi2_sum=0;
	for(i=i_l;i<=i_r;i++)
	{
		mod=as2*pow((double)i,2)+as1*i+as0;
		chi2_sum=chi2_sum+pow(mod-sp[i],2);

	}
	*a0=as0;
	*a1=as1;
	*a2=as2;
	*chi2=chi2_sum/(double)(i_r-i_l-3);
	
} 

void isqfit2(double *a2, double *a1, double *a0, double *chi2, long sp[], int i_l, int i_r)
{
	int i;
	double sum0,sumx1,sumx2,sumx3,sumx4,sumy,sumxy,sumx2y;
	double as2,as1,as0;
	double delta; 
	double chi2_sum,mod;
	
	double J[nlength][nlength];

	double L[nlength][nlength];
	double LT[nlength][nlength];	
	
	double a[nlength];
	double prava[nlength];
	double y[nlength];	

	sum0=0;
	sumx1=0;
	sumx2=0;
	sumx3=0;
	sumx4=0;
	sumy=0;
	sumxy=0;
	sumx2y=0;
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+(double)i;
		sumx2=sumx2+pow((double)i,2);
		sumx3=sumx3+pow((double)i,3);
		sumx4=sumx4+pow((double)i,4);
		sumy=sumy+sp[i];
		sumxy=sumxy+(double)i*(double)sp[i];
		sumx2y=sumx2y+pow((double)i,2)*(double)sp[i];
	}

	J[0][0]=sum0;
	J[0][1]=J[1][0]=sumx1;
	J[1][1]=J[0][2]=J[2][0]=sumx2;
	J[2][2]=sumx4;
	J[1][2]=J[2][1]=sumx3;
	prava[0]=sumy;
	prava[1]=sumxy;
	prava[2]=sumx2y;
//	printMatrix(J);
		cholesky(J, L);
//	printMatrix(L);		
		trans(L, LT);
//	printMatrix(LT);		
		solve_lower(L, y, prava);
		solve_upper(LT, a, y);
	as0=a[0];
	as1=a[1];
	as2=a[2];
	chi2_sum=0;
	for(i=i_l;i<=i_r;i++)
	{
		mod=as2*pow((double)i,2)+as1*i+as0;
		chi2_sum=chi2_sum+pow(mod-sp[i],2);

	}
	*a0=as0;
	*a1=as1;
	*a2=as2;
	*chi2=chi2_sum/(double)(i_r-i_l-3);
	
} 

void sisqfit2(double *a2, double *a1, double *a0, double *chi2, short int sp[], int i_l, int i_r)
{
	int i;
	double sum0,sumx1,sumx2,sumx3,sumx4,sumy,sumxy,sumx2y;
	double as2,as1,as0;
	double delta; 
	double chi2_sum,mod;
	
	double J[nlength][nlength];

	double L[nlength][nlength];
	double LT[nlength][nlength];	
	
	double a[nlength];
	double prava[nlength];
	double y[nlength];	

	sum0=0;
	sumx1=0;
	sumx2=0;
	sumx3=0;
	sumx4=0;
	sumy=0;
	sumxy=0;
	sumx2y=0;
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+(double)i;
		sumx2=sumx2+pow((double)i,2);
		sumx3=sumx3+pow((double)i,3);
		sumx4=sumx4+pow((double)i,4);
		sumy=sumy+sp[i];
		sumxy=sumxy+(double)i*(double)sp[i];
		sumx2y=sumx2y+pow((double)i,2)*(double)sp[i];
	}

	J[0][0]=sum0;
	J[0][1]=J[1][0]=sumx1;
	J[1][1]=J[0][2]=J[2][0]=sumx2;
	J[2][2]=sumx4;
	J[1][2]=J[2][1]=sumx3;
	prava[0]=sumy;
	prava[1]=sumxy;
	prava[2]=sumx2y;
//	printMatrix(J);
		cholesky(J, L);
//	printMatrix(L);		
		trans(L, LT);
//	printMatrix(LT);		
		solve_lower(L, y, prava);
		solve_upper(LT, a, y);
	as0=a[0];
	as1=a[1];
	as2=a[2];
	chi2_sum=0;
	for(i=i_l;i<=i_r;i++)
	{
		mod=as2*pow((double)i,2)+as1*i+as0;
		chi2_sum=chi2_sum+pow(mod-sp[i],2);

	}
	*a0=as0;
	*a1=as1;
	*a2=as2;
	*chi2=chi2_sum/(double)(i_r-i_l-3);
	
} 

void sqfit(double *a2, double *a1, double *a0, double *chi2, int sp[], int i_l, int i_r)
{
	
	int i;
	double sum0,sumx1,sumx2,sumx3,sumx4,sumy,sumxy,sumx2y;
	double delta; 
	double chi2_sum,mod;

	

	sum0=0;
	sumx1=0;
	sumx2=0;
	sumx3=0;
	sumx4=0;
	sumy=0;
	sumxy=0;
	sumx2y=0;
	
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+(double)i;
		sumx2=sumx2+pow((double)i,2);
		sumx3=sumx3+pow((double)i,3);
		sumx4=sumx4+pow((double)i,4);
		sumy=sumy+sp[i];
		sumxy=sumxy+(double)i*sp[i];
		sumx2y=sumx2y+pow((double)i,2)*sp[i];
	}

	delta=sum0*sumx2*sumx4+sumx1*sumx2*sumx3+sumx1*sumx2*sumx3-sumx2*sumx2*sumx2-sum0*sumx3*sumx3-sumx1*sumx1*sumx4;
	if(delta!=0)
	{
		*a0=(sumy*sumx2*sumx4+sumx1*sumx3*sumx2y+sumx2*sumx3*sumxy-sumx2*sumx2*sumx2y-sumx3*sumx3*sumy-sumx1*sumx4*sumxy)/delta;
		*a1=(sum0*sumx4*sumxy+sumx2*sumx3*sumy+sumx1*sumx2*sumx2y-sumx2*sumx2*sumxy-sum0*sumx3*sumx2y-sumx1*sumx4*sumy)/delta;
		*a2=(sum0*sumx2*sumx2y+sumx1*sumx2*sumxy+sumx1*sumx3*sumy-sumx2*sumx2*sumy-sum0*sumx3*sumxy-sumx1*sumx1*sumx2y)/delta;
	}
	else
	{
		*a0=0;
		*a1=0;
		*a2=0;
	}
	chi2_sum=0;
	for(i=i_l;i<=i_r;i++)
	{
		mod=*a2*pow((double)i,2)+*a1*i+*a0;
		chi2_sum=chi2_sum+pow(mod-sp[i],2);
	
	}
	*chi2=chi2_sum/(double)(i_r-i_l-3);
	
	
	
}

void sisqfit(double *a2, double *a1, double *a0, double *chi2, short int sp[], int i_l, int i_r)
{
	
	int i;
	double sum0,sumx1,sumx2,sumx3,sumx4,sumy,sumxy,sumx2y;
	double delta; 
	double chi2_sum,mod;

	sum0=0;
	sumx1=0;
	sumx2=0;
	sumx3=0;
	sumx4=0;
	sumy=0;
	sumxy=0;
	sumx2y=0;
	
	for(i=i_l;i<=i_r;i++)
	{
		sum0++;
		sumx1=sumx1+(double)i;
		sumx2=sumx2+pow((double)i,2);
		sumx3=sumx3+pow((double)i,3);
		sumx4=sumx4+pow((double)i,4);
		sumy=sumy+sp[i];
		sumxy=sumxy+(double)i*(double)sp[i];
		sumx2y=sumx2y+pow((double)i,2)*(double)sp[i];
	}

	delta=sum0*sumx2*sumx4+sumx1*sumx2*sumx3+sumx1*sumx2*sumx3-sumx2*sumx2*sumx2-sum0*sumx3*sumx3-sumx1*sumx1*sumx4;
	if(delta!=0)
	{
		*a0=(sumy*sumx2*sumx4+sumx1*sumx3*sumx2y+sumx2*sumx3*sumxy-sumx2*sumx2*sumx2y-sumx3*sumx3*sumy-sumx1*sumx4*sumxy)/delta;
		*a1=(sum0*sumx4*sumxy+sumx2*sumx3*sumy+sumx1*sumx2*sumx2y-sumx2*sumx2*sumxy-sum0*sumx3*sumx2y-sumx1*sumx4*sumy)/delta;
		*a2=(sum0*sumx2*sumx2y+sumx1*sumx2*sumxy+sumx1*sumx3*sumy-sumx2*sumx2*sumy-sum0*sumx3*sumxy-sumx1*sumx1*sumx2y)/delta;
	}
	else
	{
		*a0=0;
		*a1=0;
		*a2=0;
	}
	chi2_sum=0;
	for(i=i_l;i<=i_r;i++)
	{
		mod=*a2*pow((double)i,2)+*a1*i+*a0;
		chi2_sum=chi2_sum+pow(mod-sp[i],2);
	
	}
	*chi2=chi2_sum/(double)(i_r-i_l-3);
	
	
}

void get_simax(int *max,int *ch_max,short int sp[], int i_l, int i_r)
{
	int i;

	*max=sp[i_l];
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>*max) 
		{
			*max=sp[i];
			*ch_max=i;
		}
	}
}

void get_imax(int *max,int *ch_max,int sp[], int i_l, int i_r)
{
	int i;

	*max=sp[i_l];
	*ch_max=i_l;
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>*max) 
		{
			*max=sp[i];
			*ch_max=i;
		}
	}
}

void get_fmax(double *max,double *ch_max,int sp[], int i_l, int i_r)
{
	int i;

	*max=sp[i_l];
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>*max) 
		{
			*max=sp[i];
			*ch_max=i;
		}
	}
}

void get_dmax(double *max,double *ch_max,double sp[], int i_l, int i_r)
{
	int i;

	*max=sp[i_l];
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>*max) 
		{
			*max=sp[i];
			*ch_max=i;
		}
	}
}

void get_imax2(double *max,double *ch_max,double *chi2,int sp[], int i_l, int i_r, int step)
{

	int i;
	double a2,a1,a0;
	double rmax,rch_max;
	int il_fit,ir_fit;
	double rchi2;

	rmax=(double)sp[i_l];
	rch_max=i_l;
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>rmax) 
		{
			rmax=(double)sp[i];
			rch_max=(double)i;
		}
	}
	
	il_fit=(int)rch_max-step;
	ir_fit=(int)rch_max+step;
	if(il_fit<i_l) il_fit=i_l;
	if(ir_fit>i_r) ir_fit=i_r;
	sqfit2(&a2,&a1,&a0,&rchi2,sp,il_fit,ir_fit);
	if(a2!=0) *ch_max=-a1/(2*a2);
	else	*ch_max=rch_max;
		
	*max=a2*pow(*ch_max,2)+a1*(*ch_max)+a0;
	if(*ch_max<il_fit || *ch_max>ir_fit)
	{
		*ch_max=rch_max;
		*max=rmax;
	}
	*chi2=rchi2;
}

void get_simax2(double *max,double *ch_max,double *chi2,short int sp[], int i_l, int i_r, int step)
{
	int i;
	double a2,a1,a0;
	double rmax,rch_max;
	int il_fit,ir_fit;
	double rchi2;

	rmax=(double)sp[i_l];
	rch_max=i_l;
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>rmax) 
		{
			rmax=(double)sp[i];
			rch_max=(double)i;
		}
	}
	il_fit=(int)rch_max-step;
	ir_fit=(int)rch_max+step;
	if(il_fit<i_l) il_fit=i_l;
	if(ir_fit>i_r) ir_fit=i_r;
	sisqfit2(&a2,&a1,&a0,&rchi2,sp,il_fit,ir_fit);
	if(a2!=0) *ch_max=-a1/(2*a2);
	else	*ch_max=rch_max;
		
	*max=a2*pow(*ch_max,2)+a1*(*ch_max)+a0;
	if(*ch_max<il_fit || *ch_max>ir_fit)
	{
		*ch_max=rch_max;
		*max=rmax;
	}
	*chi2=rchi2;
}

void get_simax2p(double *max,double *ch_max,double *chi2,int *ich_wmax, double *c2, double *c1, double *c0, short int sp[], int i_l, int i_r, int step)
{
	int i;
	double a2,a1,a0;
	double rmax,rch_max;
	int il_fit,ir_fit;
	double rchi2;

	
	rmax=(double)sp[i_l];
	rch_max=i_l;
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>rmax) 
		{
			rmax=(double)sp[i];
			rch_max=(double)i;
			*ich_wmax=i;
		}
	}
	il_fit=(int)rch_max-step;
	ir_fit=(int)rch_max+step;
	if(il_fit<i_l) il_fit=i_l;
	if(ir_fit>i_r) ir_fit=i_r;
	sisqfit2(&a2,&a1,&a0,&rchi2,sp,il_fit,ir_fit);
	*c2=a2;
	*c1=a1;
	*c0=a0;
	if(a2!=0) *ch_max=-a1/(2*a2);
	else
	{
			*ch_max=rch_max;
			*c2=0.0;
			*c1=0.0;
			*c0=0.0;
	}
		
	*max=a2*pow(*ch_max,2)+a1*(*ch_max)+a0;
	if(*ch_max<il_fit || *ch_max>ir_fit)
	{
		*ch_max=rch_max;
		*max=rmax;
		*c2=0.0;
		*c1=0.0;
		*c0=0.0;
	}
	*chi2=rchi2;
}

void get_dmax(double *max,int *ch_max,double sp[], int i_l, int i_r)
{
	int i;

	*max=sp[i_l];
	for(i=i_l;i<=i_r;i++)
	{
		if(sp[i]>*max) 
		{
			*max=sp[i];
			*ch_max=i;
		}
	}
}

void get_2Dmax(int *ch_max1, int *ch_max2, int hist[][maxhist_2D])
{
	int max;
	int i,j;
	max=0;
	for(i=0;i<maxhist_2D;i++)
	{
		for(j=0;j<maxhist_2D;j++)
		{
			if(hist[i][j]>max) 
			{
				max=hist[i][j];
				*ch_max1=i;
				*ch_max2=j;
			} //if
		}//j
	}//i
}

void w_2Dcuts(int ch_max1, int ch_max2, int hist[][maxhist_2D], double range1_min, double range1_max, double range2_min, double range2_max )
{
	FILE *f;
	int i;
	double x1,x2;

	f=fopen("cut-max.txt","w");
	for(i=0;i<maxhist_2D;i++)
	{
		x1=range1_min+(double)i*(range1_max-range1_min)/(double)maxhist_2D;
		x2=range2_min+(double)i*(range2_max-range2_min)/(double)maxhist_2D;
		fprintf(f,"%8ld %18.6lf %18.6lf %8ld %8ld\n",i,x1,x2,hist[i][ch_max2],hist[ch_max1][i]); 
		
	}//i
	fclose(f);
}

void get_iarea(double *darea,int *i_l,int *i_r,int ch_max,int sp[], double treshold)
{
	int i;

	i=ch_max;
	while(sp[i]>treshold) i--;
	*i_l=i;
	i=ch_max;
	while(sp[i]>treshold) i++;
	*i_r=i;
	*darea=0.0;
	for(i=*i_l;i<=*i_r;i++) 
	{
		*darea=*darea+sp[i];
	}
}

void get_darea(double *darea,int *i_l,int *i_r,int ch_max,double sp[], double treshold)
{
	int i;

	i=ch_max;
	while(sp[i]>treshold && i>0) i--;
	if((fabs(sp[i]-treshold))<(fabs(sp[i+1]-treshold))) *i_l=i;
	else *i_l=i+1;
	i=ch_max;
	while(sp[i]>treshold && i<maxch) i++;
	if((fabs(sp[i]-treshold))<(fabs(sp[i-1]-treshold))) *i_r=i;
	else *i_r=i-1;
	*darea=0.0;
	for(i=*i_l;i<=*i_r;i++) 
	{
		*darea=*darea+sp[i];
	}
}

void get_area(double *area, short int wav[], int i_l, int i_r)
{
	int i;
	double sum;

	sum=0;
	for(i=i_l;i<=i_r;i++) sum=sum+wav[i];
	*area=sum;
}

void get_bcg(double *bcg_mean, double *bcg_sigma,short int sp[],int i_l,int i_r)
{
	int i,i_count;
	double bcg_mean_old;

	*bcg_mean=0.0;
	*bcg_sigma=0.0;
	
	for(i=i_l;i<i_r;i++)
	{
		*bcg_mean=*bcg_mean+sp[i];
	}
	*bcg_mean=*bcg_mean/(double)(i_r-i_l);
	for(i=i_l;i<i_r;i++)
	{
		*bcg_sigma=*bcg_sigma+pow(*bcg_mean-sp[i],2);
	}
	*bcg_sigma=sqrt(*bcg_sigma/(double)(i_r-i_l));
	
	bcg_mean_old=*bcg_mean;
	*bcg_mean=0.0;
	i_count=0;
	for(i=i_l;i<i_r;i++)
	{
		if(fabs(bcg_mean_old-sp[i])<(*bcg_sigma)*3) 
		{
			*bcg_mean=*bcg_mean+sp[i];
			i_count++;
		}
	}

	*bcg_mean=*bcg_mean/(double)(i_count);
	//printf("%lf %lf \n",bcg_mean_old,*bcg_mean);
		
}

void get_chi2(double *chi2, double bcg, short int sp[],int i_l,int i_r)
{
	int i;

	*chi2=0.0;
	
	for(i=i_l;i<=i_r;i++)
	{
		*chi2=*chi2+pow(sp[i]-bcg,2);
	}
	*chi2=*chi2/(double)(i_r-i_l-1);
}

void get_res(double *res, double bcg, short int sp[],int i_l,int i_r)
{
	int i;
	double dif;

	*res=0.0;
	
	for(i=i_l;i<=i_r;i++)
	{
		dif=bcg-sp[i];
		if(fabs(dif)>*res) *res=dif;
	}
	
}

void add_hist(double value,int hist[],double hmin,double hmax,int n_bins)
{
	double rpom;
	int k_hist;
	
	rpom=(double)(value-hmin)/(double)(hmax-hmin)*(double)n_bins;
	k_hist=(int)rpom;
	if((rpom-k_hist)>=0.5) k_hist++;
	if((k_hist>=0)&&(k_hist<n_bins)) hist[k_hist]++;
}

void add_ihist(int value,int hist[],double hmin,double hmax,int n_bins)
{
	double rpom;
	int k_hist;
	
	rpom=(double)(value-hmin)/(double)(hmax-hmin)*(double)n_bins;
	k_hist=(int)rpom;
	if((rpom-k_hist)>=0.5) k_hist++;
	if((k_hist>=0)&&(k_hist<n_bins)) hist[k_hist]++;
}

void add_2Dhist(double value1,double value2,int hist[][maxhist_2D],double hmin,double hmax,int n_bins)
{
	double rpom1,rpom2;
	int k_hist1,k_hist2;
	
	rpom1=(double)(value1-hmin)/(double)(hmax-hmin)*(double)n_bins;
	rpom2=(double)(value2-hmin)/(double)(hmax-hmin)*(double)n_bins;
	k_hist1=(int)rpom1;
	k_hist2=(int)rpom2;
	if((rpom1-k_hist1)>=0.5) k_hist1++;
	if((rpom2-k_hist2)>=0.5) k_hist2++;
	if((k_hist1>=0)&&(k_hist1<n_bins)&&(k_hist2>=0)&&(k_hist2<n_bins)) hist[k_hist1][k_hist2]++;
}

void w_hist(int sp1[],int sp2[],char file_name[],double h_min1,double h_max1,double h_min2,double h_max2,int n_bins)
{
	FILE *fhist;
	int i;
	
	fhist=fopen(file_name,"w");
	for(i=0;i<n_bins;i++)
	{
		fprintf(fhist,"%8d %15.8lf %8d %15.8lf %8d\n",i,h_min1+(double)i*(h_max1-h_min1)/(double)n_bins,sp1[i],h_min2+(double)i*(h_max2-h_min2)/(double)n_bins,sp2[i]);
	}//i
	fclose(fhist);
}

void w_2Dhist(int sp[][maxhist_2D],char file_name[],double h_min,double h_max,int n_bins)
{
	FILE *fhist;
	int i,j;
	
	fhist=fopen(file_name,"w");
	if(h_max-h_min!=0)
	{
	for(i=0;i<n_bins;i++)
	{
		for(j=0;j<n_bins;j++) fprintf(fhist,"%15.8lf %15.8lf %8d\n",h_min+(double)i*(h_max-h_min)/(double)n_bins,h_min+(double)j*(h_max-h_min)/(double)n_bins,sp[i][j]);
	}//i
	}
	fclose(fhist);
}

void w_2Dhist2(int sp[][maxhist_py],char file_name[],double h_min,double h_max,int n_binsx, int n_binsy)
{
	FILE *fhist;
	int i,j;
	
	fhist=fopen(file_name,"w");
	if(h_max-h_min!=0)
	{
	for(i=0;i<n_binsx;i++)
	{
		for(j=0;j<n_binsy;j++) fprintf(fhist,"%15.8lf %15.8lf %8d\n",h_min+(double)i*(h_max-h_min)/(double)n_binsy,h_min+(double)j*(h_max-h_min)/(double)n_binsy,sp[i][j]);
	}//i
	}
	fclose(fhist);
}

void w_wav(int sp1[],int sp2[],char file_name[],int n_points)
{
	FILE *fsp;
	int i;
	
	fsp=fopen(file_name,"w");
	for(i=0;i<n_points;i++)
	{
		fprintf(fsp,"%8d %8d %8d \n",i,sp1[i],sp2[i]);
	}//i
	fclose(fsp);
}

void get_w(double *w,int hist[],double max, int ch_max)
{
	FILE *fpom;
	int i;
	int i_trial;
	double max_pul;
	double rch_pul_l,rch_pul_r;
	double x1,x2;
	double a2,a1,a0,d,chi2;
	int i_l,i_r;

	
	*w=0;
	max_pul=(double)max/2;
	i=ch_max;
	
	while(hist[i]>=max_pul && i>0) i--;
	i_l=i;
	rch_pul_l=i_l;
	sqfit2(&a2,&a1,&a0,&chi2,hist,i_l-5,i_l+5);
	d=a1*a1-4*a2*(a0-max_pul);
	if(d>=0 && a2!=0) 
	{
		x1=(-a1-sqrt(d))/(2*a2);
		x2=(-a1+sqrt(d))/(2*a2);
		if(fabs(x1-i_l)<fabs(x2-i_l)) rch_pul_l=x1;
		else rch_pul_l=x2;
	}

	i=ch_max;
	while(hist[i]>=max_pul && i<maxhist) i++;
	i_r=i;
	rch_pul_r=i_r;
	sqfit2(&a2,&a1,&a0,&chi2,hist,i_r-5,i_r+5);
	d=a1*a1-4*a2*(a0-max_pul);
	if(d>=0 && a2!=0) 
	{
		x1=(-a1-sqrt(d))/(2*a2);
		x2=(-a1+sqrt(d))/(2*a2);
		if(fabs(x1-i_l)<fabs(x2-i_l)) rch_pul_r=x1;
		else rch_pul_r=x2;
	}
	
	*w=fabs(rch_pul_r-rch_pul_l);
		
}

void get_wl(double *w,int hist[],double max, int ch_max)
{
	int i;
	int i_trial;
	double max_pul;
	double rch_pul;
	double w_l;
	double x1,x2;
	double a2,a1,a0,d,chi2;
	int i_l,i_r;

	max_pul=(double)max/10;
	i=ch_max;
	*w=0;
		
	while(hist[i]>=max_pul && i>0) 	i--;
	///x1=(double)(ch_max-i);
	//x2=(double)(ch_max-i+1);
	//y1=(double)hist[i];
	//y2=(double)hist[i+1];
	//w_l=x1+(max_pul-y1)*(x2-x1)/(y2-y1);
	//i_l=i-2;
	//i_r=i+5;
	//if(i_r>ch_max) i_r=ch_max;
	//ilfit(&a,&b,&chi2,hist,i_l,i_r);
	//w_l=ch_max-(max_pul-b)/a;
	//w_l=ch_max-(-b)/a;
	//if(w_l<ch_max-i_l || w_l>ch_max-i_r) w_l=x1+(max_pul-y1)*(x2-x1)/(y2-y1);
	//*w=ch_max-i;
	i_trial=i;
	i_l=i_trial-5;
	i_r=i_trial+5;
	sqfit2(&a2,&a1,&a0,&chi2,hist,i_l,i_r);
	d=a1*a1-4*a2*(a0-max_pul);
	if(d>=0 && a2!=0) 
	{
		x1=(-a1-sqrt(d))/(2*a2);
		x2=(-a1+sqrt(d))/(2*a2);
		if(fabs(x1-i_trial)<fabs(x2-i_trial)) rch_pul=x1;
		else rch_pul=x2;
	}
	else
	{
		rch_pul=(double)i_trial;
	}
	if(rch_pul>=ch_max) rch_pul=(double)i_trial;
	*w=ch_max-rch_pul;
	
}

void get_wr(double *w,int hist[],double max, int ch_max)
{
	int i;
	int i_trial;
	double max_pul;
	double rch_pul;
	double w_r;
	double x1,x2;
	double a2,a1,a0,d,chi2;
	int i_l,i_r;


	max_pul=(double)max/10;
	i=ch_max;
	*w=0;
	
	while(hist[i]>=max_pul && i<maxhist) i++;
	//x1=(double)(i-1-ch_max);
	//x2=(double)(i-ch_max);
	//y1=(double)hist[i-1];
	//y2=(double)hist[i];
	//w_r=x1+(max_pul-y1)*(x2-x1)/(y2-y1);
	//i_l=i-5;
	//i_r=i+2;
	//if(i_l<ch_max) i_l=ch_max;
	//ilfit(&a,&b,&chi2,hist,i_l,i_r);
	//w_r=(max_pul-b)/a-ch_max;
	//w_r=(-b)/a-ch_max;
	//if(w_r<i_l-ch_max || w_r>i_r-ch_max) w_r=x1+(max_pul-y1)*(x2-x1)/(y2-y1);
	
	i_trial=i;
	i_l=i_trial-5;
	i_r=i_trial+5;
	sqfit2(&a2,&a1,&a0,&chi2,hist,i_l,i_r);
	d=a1*a1-4*a2*(a0-max_pul);
	if(d>=0 && a2!=0) 
	{
		x1=(-a1-sqrt(d))/(2*a2);
		x2=(-a1+sqrt(d))/(2*a2);
		if(fabs(x1-i_trial)<fabs(x2-i_trial)) rch_pul=x1;
		else rch_pul=x2;
	}
	else
	{
		rch_pul=(double)i_trial;
	}
	if(rch_pul<=ch_max) rch_pul=(double)i_trial;
	*w=rch_pul-ch_max;
	
		
	
}

int mround(double r)
{
	int ir;

	ir=(int)r;
	if(fabs(ir-r)>0.5) ir++;
	return(ir);
}

void get_calib(double *peak,double *w,double pmin, double pmax, double hmin, double hmax,int hist[maxhist])
{
	
	//int ipeak_max;
	double peak_max;
	int ipeak_max;
	double rch_peak;
	double rch_peak_15,rch_peak_5; 
	int ch_peak;
	double peak_w;
	double chi2;
	int ch_min,ch_max;

	
	ch_min=(int)((pmin-hmin)/(hmax-hmin)*maxhist);
	ch_max=(int)((pmax-hmin)/(hmax-hmin)*maxhist);
	//get_imax(&ipeak_max,&ch_peak,hist,ch_min,ch_max);
	get_imax2(&peak_max,&rch_peak,&chi2,hist,ch_min,ch_max,15);
	rch_peak_15=rch_peak;
	get_imax2(&peak_max,&rch_peak,&chi2,hist,ch_min,ch_max,5);
	rch_peak_5=rch_peak;
	if(fabs(rch_peak_15-rch_peak_5)>15) 
		{
			get_imax(&ipeak_max,&ch_peak,hist,ch_min,ch_max);
			peak_max=(double)ipeak_max;
			rch_peak=(double)ch_peak;
	}
	ch_peak=(int)rch_peak;
	get_w(&peak_w,hist,peak_max,ch_peak);
	//*peak=hmin+(double)ch_peak*(hmax-hmin)/(double)maxhist;
	*peak=hmin+rch_peak*(hmax-hmin)/(double)maxhist;
	*w=(double)peak_w*(hmax-hmin)/(double)maxhist;
	
}

void get_wdif(int wdif[maxch], short int wav[maxch], int dif, int n_points)
{
	int i,k;

	for(i=0;i<n_points;i++) wdif[i]=0; 	

	for(i=0;i<n_points-dif;i++) 	
	{
		for(k=0;k<dif;k++)	wdif[i]=wdif[i]+wav[i+k+1]-wav[i+k];
						
	}//i	

	for(i=0;i<n_points-dif;i++) 	
	{
		for(k=0;k<dif;k++)	wdif[i]=wdif[i]+wav[i+k+1]-wav[i+k];
						
	}
}	

void smooth(int wav[],int step,int n_points)
{
	int i,k;
	//smoothing
	
	for(i=0;i<n_points-step;i++) 	
	{
		for(k=1;k<=step;k++) wav[i]=wav[i]+wav[i+k];
		wav[i]=wav[i]/20;
				
	}// i
	for(i=n_points-step;i<n_points;i++) 	
	{	
		for(k=1;k<=20;k++)	wav[i]=wav[i]+wav[i-k];
		wav[i]=wav[i]/20;
	} //i 
}

void dsmooth(double wav[],int step,int n_points)
{
	int i,k;
	double swav[maxch];
	//smoothing
	
	for(i=0;i<n_points;i++) swav[i]=wav[i]; 	
	for(i=step;i<n_points-step;i++) 	
	{
		for(k=1;k<=step;k++) swav[i]=swav[i]+wav[i+k]+wav[i-k];
		wav[i]=swav[i]/(double)(2*step+1);
				
	}// i
	
}

void shift(double wav[maxch], int step, double bcg_l,double bcg_r,int n_points)
{
	int i;


		if(step>=0) //posun vlevo
		{
			for(i=0;i<n_points-step;i++)	wav[i]=wav[i+step];
			for(i=n_points-step;i<n_points;i++) wav[i]=bcg_r;
		}
		else //posun vpravo
		{
			for(i=n_points-1;i>=-step;i--) wav[i]=wav[i+step];
			for(i=0;i<-step;i++)	wav[i]=bcg_l;
		}
}

void get_t(double *t, double f,short int wav[maxch],int ch_max, int ch_min)
{
	int i;
	double max_pul;
	double x1,x2,y1,y2;

	max_pul=(double)wav[ch_max]*f;
	i=ch_max;
	while(wav[i]>max_pul && i>ch_min) i--;
	x1=(double)i;
	x2=(double)i+1;
	y1=(double)wav[i];
	y2=(double)wav[i+1];
	if(y1==y2) *t=(x2+x1)/2;
	else *t=x1+(max_pul-y1)*(x2-x1)/(y2-y1);
	
}

void get_pulse(double pulse[], short int wav[],double wavs[], double wavr[], double bcg, int tf,int n_points)
{
	
	int i,k;
	
	for(i=0;i<n_points;i++) 
	{
		wavs[i]=(double)wav[i]-bcg;
		wavr[i]=0.0;

	}
	

	for(i=0;i<n_points;i++)
	{
		if((2*tf-i >0) && (2*tf-i <n_points)) wavr[i]=wavs[2*tf-i];
	}
	for(k=0;k<n_points;k++)
	{
		pulse[k]=0.0;
		for(i=0;i<n_points;i++)
		{
			if(k-i>0 && k-i<n_points) pulse[k]=pulse[k]+(wavr[i])*(wavs[k-i]);
				
		}
	}

}

void get_pulse2(double pulse[], short int wav[],double bcg, int tf,int n_points)
{
	

	int i,j,k;
	double wavs[maxch];
	double wavr[maxch];
	
	

	if (wavs==NULL) 
	{
		printf("not ehougt memory\n");
		exit(0);
	}
	if (wavr==NULL) 
	{
		printf("not ehougt memory\n");
		exit(0);
	}

	
	for(i=0;i<n_points;i++) 
	{
		wavs[i]=(double)wav[i]-bcg;
		wavr[i]=(double)wav[n_points-i-1]-bcg;
	
	}
	
	
	for(k=-n_points+1;k<n_points;k++)
	{
		j=k+n_points-1;
		pulse[j]=0.0;
		for(i=0;i<n_points;i++)
		{
			if(i-k>0 && i-k<n_points) pulse[j]=pulse[j]+(wavr[i-k])*(wavs[i]);
				
		}
	}

}	
	
void get_plim(double *wl, double *wm, double *wr, int hpeak[])
{
	int i;
	double sum,psum;
	double ax,bx,ay,by;
	double t;


	sum=0;
	for(i=0;i<maxhist_px;i++) sum=sum+hpeak[i];
	
	psum=0;
	i=0;
	while((psum=psum+hpeak[i])/sum<0.05) i++;
	ay=psum;
	by=psum-hpeak[i];
	ax=(double)i;
	bx=(double)i-1;
	t=(0.05-by)*(ax-bx)/(ay-by);
	*wl=t;
	
	psum=0;
	i=0;
	while((psum=psum+hpeak[i])/sum<0.5) i++;
	ay=psum;
	by=psum-hpeak[i];
	ax=(double)i;
	ay=(double)i-1;
	t=(0.5-by)*(ax-bx)/(ay-by);
	*wm=t;

	psum=0;
	i=0;
	while((psum=psum+hpeak[i])/sum<0.95) i++;
	ay=psum;
	by=psum-hpeak[i];
	ax=(double)i;
	ay=(double)i-1;
	t=(0.95-by)*(ax-bx)/(ay-by);
	*wr=t;

}



void main()
{
	FILE *fin;
	FILE *fsp;
	FILE *fstab;
	FILE *fcal;
	FILE *fcal1;
	FILE *fwav1;
	FILE *fwav2;
	FILE *fwdif1;
	FILE *fwdif2;
	FILE *fpeakm;
	FILE *fstat;
	FILE *fpeak_mean;
	FILE *fpeakh;
	FILE *febin;
	FILE *fparabola1;
	FILE *fparabola2;
	FILE *flog;

	
	
	int i; //pro points
	int j; //pro waveforms
	int l; //pro sessions
	int k;
	
	//int k_hist;
	//int i_good;
	int i_good1,i_good2;
	int n_all,n_good;
	int n_good1,n_good2;
	int i_mode;
	int text_mode;

    //
	int n_points;
	int n_wav;
	int n_sess;

	char c;
	char  sp_path[80];
	char sp_file[80];

	int r_nbrSamples;
	int r_nbrSegments;
	int r_nbrWaveforms;

	//waveforms
	static short int wav1[maxch]; 
	static short int wav2[maxch];
	static int wdif1[maxch];
	static int wdif2[maxch];
	static double wav1_n[maxch];
	static double wav2_n[maxch];
	int ref_point1,ref_point2;
	int raw_divider;
			
	//darea
	double darea1_min,darea1_max;
	double darea2_min,darea2_max;
	double darea1,darea2;
	int ch_darea1_l,ch_darea1_r;
	int ch_darea2_l,ch_darea2_r;
	static int hdarea1[maxhist_small];
	static int hdarea2[maxhist_small];
	int ch_darea1,ch_darea2;
	static int hch_darea1[maxhist_small2],hch_darea2[maxhist_small2];
	double hch_darea1_min,hch_darea1_max;
	double hch_darea2_min,hch_darea2_max;
	double ch_darea1_l_limit,ch_darea1_r_limit;
	double ch_darea2_l_limit,ch_darea2_r_limit;
	double wdif1_treshold,wdif2_treshold;;
	

	//wdif_max
	double wdif1_max;
	double wdif2_max;
	double ch_wdif1_max;
	double ch_wdif2_max;
	double wdif1_max_min,wdif1_max_max;
	double wdif2_max_min,wdif2_max_max;
	static int hwdif1_max[maxhist_small];
	static int hwdif2_max[maxhist_small];
	int wdif1_max_tolerance,wdif2_max_tolerance;
	int wdif1_smooth,wdif2_smooth;

	//pmax
	double pmax1;
	double pmax2;
	double pmax1_min,pmax1_max;
	double pmax2_min,pmax2_max;
	static int hpmax1[maxhist];
	static int hpmax2[maxhist];
	double ch_pmax1;
	double ch_pmax2;

	//bdif
	static int hbdif1[maxhist];
	static int hbdif2[maxhist];
	double bdif1_min,bdif1_max;
	double bdif2_min,bdif2_max;
	int ch_wmax1;
	double wmax1,fch_wmax1;
	int ch_wmax2;
	double wmax2,fch_wmax2;
	static int hwmax1[maxhist],hch_wmax1[maxhist_small2];
	static int hwmax2[maxhist],hch_wmax2[maxhist_small2];
	double hwmax1_min,hwmax1_max;
	double hwmax2_min,hwmax2_max;
	double hch_wmax1_min,hch_wmax1_max;
	double hch_wmax2_min,hch_wmax2_max;
	double ch_wmax1_l_limit,ch_wmax1_r_limit;
	double ch_wmax2_l_limit,ch_wmax2_r_limit;
	double bcg1_mean_l,bcg1_sigma_l;
	double bcg2_mean_l,bcg2_sigma_l;
	double bcg1_mean_l_min,bcg1_mean_l_max;
	double bcg2_mean_l_min,bcg2_mean_l_max;
	double bcg1_sigma_l_min,bcg1_sigma_l_max;
	double bcg2_sigma_l_min,bcg2_sigma_l_max;
	double bdif1,bdif2;
	static int hbcg1_mean_l[maxhist_small];
	static int hbcg2_mean_l[maxhist_small];
	static int hbcg1_sigma_l[maxhist_small];
	static int hbcg2_sigma_l[maxhist_small];
	double bcg1_mean_l_l_limit;
	double bcg2_mean_l_l_limit;
	double bcg1_mean_l_l_limit_rel,bcg1_mean_l_r_limit_rel;
	double bcg2_mean_l_l_limit_rel,bcg2_mean_l_r_limit_rel;
	double bcg1_sigma_l_l_limit,bcg1_sigma_l_r_limit;
	double bcg2_sigma_l_l_limit,bcg2_sigma_l_r_limit;
	double chi2l_1,chi2l_2;
	static int hchi2l_1[maxhist_small];
	static int hchi2l_2[maxhist_small];
	double chi2l_1_min,chi2l_1_max;
	double chi2l_2_min,chi2l_2_max;
	double chi2l_1_l,chi2l_1_r;
	double chi2l_2_l,chi2l_2_r;
	static int hchi2sq1[maxhist_small];
	static int hchi2sq2[maxhist_small];
	double chi2sq1_min,chi2sq1_max;
	double chi2sq2_min,chi2sq2_max;
	double chi2sq1,chi2sq2;
	double chi2sq1_l,chi2sq1_r;
	double chi2sq2_l,chi2sq2_r;
	int ich_wmax1,ich_wmax2;
	double c2,c1,c0;
	int wmax_fit_range;

	//bdif_cal
	double bdif1_peak1,bdif1_peak1_avg;
	double bdif2_peak1,bdif2_peak1_avg;
	double bdif1_peak1_min,bdif1_peak1_max;
	double bdif2_peak1_min,bdif2_peak1_max;
	double bdif1_cal_min,bdif1_cal_max;
	double w1_peak1;
	double w2_peak1;
	double bdif1_peak2,bdif1_peak2_avg;
	double bdif2_peak2,bdif2_peak2_avg;
	double bdif1_peak2_min,bdif1_peak2_max;
	double bdif2_peak2_min,bdif2_peak2_max;
	double bdif2_cal_min,bdif2_cal_max;
	double w1_peak2;
	double w2_peak2;
	double a1_cal,b1_cal;
	double a2_cal,b2_cal;
	double a1_cal_avg,b1_cal_avg;
	double a2_cal_avg,b2_cal_avg;
	static int hbdif1_cal[maxhist];
	static int hbdif2_cal[maxhist];
	double bdif1_cal;
	double bdif2_cal;
	double cal_energy1,cal_energy2;
	int avg_cal;
	

	//watchdogs
	double wmax1_l,wmax1_r; 
	double wmax2_l,wmax2_r;
	int iwmax_wdog;
	int ch_wmax1_l,ch_wmax1_r; 
	int ch_wmax2_l,ch_wmax2_r;
	int ich_wmax_wdog;
	int ichi2sq_wdog;
	int ibcg_sigma_l_wdog;

	//stability
	static double bcg1_sess[max_sess];
	static double bcg2_sess[max_sess];
	int ipom;
	double rpom;
	static double ch_bcg1_sess[max_sess];
	static double ch_bcg2_sess[max_sess];
	static double bdif1_sess[max_sess];
	static double bdif2_sess[max_sess];
	double bdif1_sess_avg;
	double bdif2_sess_avg;
	static double ch_bdif1_max[max_sess];
	static double ch_bdif2_max[max_sess];
	static int hbcg1_mean_l_sess[maxhist_small];
	static int hbcg2_mean_l_sess[maxhist_small];
	static int hbdif1_sess[maxhist];
	static int hbdif2_sess[maxhist];
	static double fbdif1_sess[max_sess];
	static double fbdif2_sess[max_sess];
	double chi2_pom;
	
	
	//timing
	static int ht1[maxhist_small];
	static int ht2[maxhist_small];
	double t1,t2;
	double t1_min,t1_max;
	double t2_min,t2_max;
	double t1_l,t1_r;
	double t2_l,t2_r;
	static int hdt[maxhist_small];
	double dt;
	double dt_min,dt_max;
	double dt_l,dt_r;
	double frac1,frac2;
	int shift1,shift2;

	//ee
	static int hrf[maxhist_2D];
	static int hdb[maxhist_2D];
	double rf,db;
	double hrf_min,hrf_max;
	double hdb_min,hdb_max;
	double rf_delta,db_delta;
	int ch_max_db,ch_max_rf;

	//2D histogram
	int (*ee)[maxhist_2D];
	
	
	//peakm
	int (*pp1)[maxhist_py];
	int (*pp2)[maxhist_py];
	static double peak1m[maxhist_px];
	static double peak2m[maxhist_px];
	static double peak1l[maxhist_px];
	static double peak2l[maxhist_px];
	static double peak1r[maxhist_px];
	static double peak2r[maxhist_px];
	double peak1_max;
	double peak2_max;
	double ch_peak1m;
	double ch_peak2m;
	double chi2_peak1m;
	double chi2_peak2m;
	double peak1m_min,peak1m_max;
	double peak2m_min,peak2m_max;
	double wpeak1_l,wpeak1_r;
	double wpeak2_l,wpeak2_r;
	double f_ext_l,f_ext_r;
	static double pp1_mean[maxch];
	static double pp2_mean[maxch];
	int pp1_counter;
	int pp2_counter;
	double psumxx,psumxy;
	int shift_offset;
	
	//annihilation in flight
	double aif_min,aif_max;
	double d_aif;
	static int haif[maxhist];

	//time
	 time_t rawtime;
	struct tm *timeinfo;
	clock_t start_session,stop_session;
	double t_session;

	//suffixes
	char suffix_m1[5];
	char suffix_0[5];
	char suffix_1[5];
	char suffix_2[5];
	char suffix_3[5];
	char suffix_4[5];
	char suffix_5[5];
	
	flog=fopen("cmfit.log","w");

	printf("Program cmfit12 - analysis of waveforms from digital CDB measurement\n");
	fprintf(flog,"Program cmfit12 - analysis of waveforms from digital CDB measurement\n");
	printf("version 1.00 5.5.2012\n");
	fprintf(flog,"version 1.00 5.5.2012\n");
	printf("copyright: Jakub Cizek\n");
	fprintf(flog,"copyright: Jakub Cizek\n");
	printf("Faculty of Mathematics and Physics, Charles University in Prague\n");
	fprintf(flog,"Faculty of Mathematics and Physics, Charles University in Prague\n");
	time(&rawtime);
	timeinfo=localtime(&rawtime);
	printf("%s", asctime(timeinfo));
	fprintf(flog,"%s", asctime(timeinfo));
	printf("\n");
	fprintf(flog,"\n");

	
	//reading ini file cmfit.ini
	if(fexist("cmfit.ini")) fin=fopen("cmfit.ini","r");
	else exit(0);
	//mode
	printf("settings from cmfit.ini:\n"); 
	fprintf(flog,"settings from cmfit.ini:\n"); 
	fscanf(fin,"%d",&i_mode);
	while((c=getc(fin))!='\n');
	if(i_mode==-1) 
	{
			printf("mode: %d - wf write\n",i_mode);
			fprintf(flog,"mode: %d - wf write\n",i_mode);
	}
	if(i_mode==0) 
	{
			printf("mode: %d - single raw\n",i_mode);
			fprintf(flog,"mode: %d - single raw\n",i_mode);
	}
	if(i_mode==1) 
	{
			printf("mode: %d - single filters\n",i_mode);
			fprintf(flog,"mode: %d - single filters\n",i_mode);
	}
	if(i_mode==2) 
	{
			printf("mode: %d - CDB raw\n",i_mode);
			fprintf(flog,"mode: %d - CDB raw\n",i_mode);
	}
	if(i_mode==3) 
	{
			printf("mode: %d - CDB e-cal-avg\n",i_mode);
			fprintf(flog,"mode: %d - CDB e-cal-avg\n",i_mode);
	}
	if(i_mode==4) 
	{
			printf("mode: %d - CDB filters\n",i_mode);
			fprintf(flog,"mode: %d - CDB filters\n",i_mode);
	}
	if(i_mode==5) 
	{
			printf("mode: %d - CDB filters e-cal-avg\n",i_mode);
			fprintf(flog,"mode: %d - CDB filters e-cal-avg\n",i_mode);
	}
	//n_points
	fscanf(fin,"%d",&n_points);
	while((c=getc(fin))!='\n');
	printf("number of points: %d\n",n_points);
	fprintf(flog,"number of points: %d\n",n_points);
	if(n_points>maxch) 
	{
		printf("Error: number of poins must be smaller than 10000\n"); 
		exit(0);
	}
	//n_waveforms
	fscanf(fin,"%d",&n_wav);
	while((c=getc(fin))!='\n');
	printf("number of waveforms to read: %d\n",n_wav);
	fprintf(flog,"number of waveforms to read: %d\n",n_wav);
	//n_sessions
	fscanf(fin,"%d",&n_sess);
	while((c=getc(fin))!='\n');
	printf("number of sessions to read: %d\n",n_sess);
	fprintf(flog,"number of sessions to read: %d\n",n_sess);
	//file path
	fscanf(fin,"%s",sp_path);
	printf("spectrum file: %s\n",sp_path);
	fprintf(flog,"spectrum file: %s\n",sp_path);
	//calibration
	fscanf(fin,"%d",&avg_cal);
	while((c=getc(fin))!='\n');
	if(avg_cal==1)
		{
			printf("average calibration used\n");
			fprintf(flog,"average calibration used\n");
	}
	else 
		{
			printf("independent calibration for each session\n");
			fprintf(flog,"independent calibration for each session\n");
	}
	fscanf(fin,"%d,%d",&ref_point1,&ref_point2);
	while((c=getc(fin))!='\n');
	printf("referece point ch1: %d, ch2: %d\n",ref_point1,ref_point2);
	fprintf(flog,"referece point ch1: %d, ch2: %d\n",ref_point1,ref_point2);
	fscanf(fin,"%lf,%lf",&wdif1_treshold,&wdif2_treshold);
	while((c=getc(fin))!='\n');
	printf("wdif_tresholds: %lf, %lf\n",wdif1_treshold,wdif2_treshold);
	fprintf(flog,"wdif_tresholds: %lf, %lf\n",wdif1_treshold,wdif2_treshold);
	fscanf(fin,"%d,%d",&wdif1_smooth,&wdif2_smooth);
	while((c=getc(fin))!='\n');
	printf("wdif_smoothing ch1:%d, ch2:%d\n",wdif1_smooth,wdif2_smooth);
	fprintf(flog,"wdif_smoothing ch1:%d, ch2:%d\n",wdif1_smooth,wdif2_smooth);
	fscanf(fin,"%d",&raw_divider);
	while((c=getc(fin))!='\n');
	printf("raw divider: %d\n",raw_divider);
	fprintf(flog,"raw divider: %d\n",raw_divider);
	fscanf(fin,"%d",&shift_offset);
	while((c=getc(fin))!='\n');
	printf("wf shift offset: %d\n",shift_offset);
	fprintf(flog,"wf shift offset: %d\n",shift_offset);
	fscanf(fin,"%d",&wmax_fit_range);
	while((c=getc(fin))!='\n');
	printf("wmax fit range: %d\n",wmax_fit_range);
	fprintf(flog,"wmax fit range: %d\n",wmax_fit_range);
	fscanf(fin,"%lf,%lf",&frac1,&frac2);
	while((c=getc(fin))!='\n');
	printf("CFD fractions: %lf, %lf\n",frac1,frac2);
	fprintf(flog,"CFD fractions: %lf, %lf\n",frac1,frac2);
	fscanf(fin,"%lf,%lf",&rf_delta,&db_delta);
	while((c=getc(fin))!='\n');
	printf("delta in cuts RF: %lf, DB: %lf\n",rf_delta,db_delta);
	fprintf(flog,"delta in cuts RF: %lf, DB: %lf\n",rf_delta,db_delta);
	fscanf(fin,"%lf,%lf",&hrf_min,&hrf_max);
	while((c=getc(fin))!='\n');
	printf("RF histogram min: %lf, max: %lf\n",hrf_min,hrf_max);
	fprintf(flog,"RF histogram min: %lf, max: %lf\n",hrf_min,hrf_max);
	fscanf(fin,"%lf,%lf",&hdb_min,&hdb_max);
	while((c=getc(fin))!='\n');
	printf("DB histogram min: %lf, max: %lf\n",hdb_min,hdb_max);
	fprintf(flog,"DB histogram min: %lf, max: %lf\n",hdb_min,hdb_max);
	fscanf(fin,"%lf,%lf",&aif_min,&aif_max);
	while((c=getc(fin))!='\n');
	printf("AIF histogram min: %lf (keV), max: %lf (keV)\n",aif_min,aif_max);
	fprintf(flog,"AIF histogram min: %lf (keV), max: %lf (keV)\n",aif_min,aif_max);
	fscanf(fin,"%lf",&d_aif);
	while((c=getc(fin))!='\n');
	printf("delta for AIF: %lf (keV)\n",d_aif);
	fprintf(flog,"delta for AIF: %lf (keV)\n",d_aif);
	fscanf(fin,"%lf,%lf",&f_ext_l,&f_ext_r);
	while((c=getc(fin))!='\n');
	printf("Extension of filters lower: %lf, upper: %lf\n",f_ext_l,f_ext_r);
	fprintf(flog,"Extension of filters lower: %lf, upper: %lf\n",f_ext_l,f_ext_r);
	fscanf(fin,"%d",&text_mode);
	printf("Text mode: ");
	fprintf(flog,"Text mode: ");
	if(text_mode==0) 
		{
			printf(" %d-quiet\n",text_mode);
			fprintf(flog," %d-quiet\n",text_mode);
	}
	if(text_mode==1) 
		{
			printf(" %d-verbose\n",text_mode);
			fprintf(flog," %d-verbose\n",text_mode);
	}
	fclose(fin);

	//histograms
	//reading file histogram.ini
	if(text_mode==1) 
	{
		printf("\n"); 
		printf("histograms:\n"); 
	}
	fprintf(flog,"\n"); 
	fprintf(flog,"histograms:\n"); 
	if(fexist("histograms.ini")) fin=fopen("histograms.ini","r");
	else exit(0);
	//energy spectrum
	fscanf(fin,"%lf,%lf",&bdif1_cal_min,&bdif1_cal_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bdif1_cal min: %lf, max: %lf\n",bdif1_cal_min,bdif1_cal_max);
	fprintf(flog,"histogram bdif1_cal min: %lf, max: %lf\n",bdif1_cal_min,bdif1_cal_max);
	fscanf(fin,"%lf,%lf",&bdif2_cal_min,&bdif2_cal_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bdif2_cal min: %lf, max: %lf\n",bdif2_cal_min,bdif2_cal_max);
	fprintf(flog,"histogram bdif2_cal min: %lf, max: %lf\n",bdif2_cal_min,bdif2_cal_max);
	fscanf(fin,"%lf,%lf",&darea1_min,&darea1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram darea1 min: %lf, max: %lf\n",darea1_min,darea1_max);
	fprintf(flog,"histogram darea1 min: %lf, max: %lf\n",darea1_min,darea1_max);
	fscanf(fin,"%lf,%lf",&darea2_min,&darea2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram darea2 min: %lf, max: %lf\n",darea2_min,darea2_max);
	fprintf(flog,"histogram darea2 min: %lf, max: %lf\n",darea2_min,darea2_max);
	fscanf(fin,"%lf,%lf",&wdif1_max_min,&wdif1_max_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram wdif_max1 min: %lf, max: %lf\n",wdif1_max_min,wdif1_max_max);
	fprintf(flog,"histogram wdif_max1 min: %lf, max: %lf\n",wdif1_max_min,wdif1_max_max);
	fscanf(fin,"%lf,%lf",&wdif2_max_min,&wdif2_max_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram wdif_max2 min: %lf, max: %lf\n",wdif2_max_min,wdif2_max_max);
	fprintf(flog,"histogram wdif_max2 min: %lf, max: %lf\n",wdif2_max_min,wdif2_max_max);
	fscanf(fin,"%lf,%lf",&bcg1_mean_l_min,&bcg1_mean_l_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bcg1_mean_l min: %lf, max: %lf\n",bcg1_mean_l_min,bcg1_mean_l_max);
	fprintf(flog,"histogram bcg1_mean_l min: %lf, max: %lf\n",bcg1_mean_l_min,bcg1_mean_l_max);
	fscanf(fin,"%lf,%lf",&bcg2_mean_l_min,&bcg2_mean_l_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bcg2_mean_l min: %lf, max: %lf\n",bcg2_mean_l_min,bcg2_mean_l_max);
	fprintf(flog,"histogram bcg2_mean_l min: %lf, max: %lf\n",bcg2_mean_l_min,bcg2_mean_l_max);
	fscanf(fin,"%lf,%lf",&bcg1_sigma_l_min,&bcg1_sigma_l_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bcg1_sigma_l min: %lf, max: %lf\n",bcg1_sigma_l_min,bcg1_sigma_l_max);
	fprintf(flog,"histogram bcg1_sigma_l min: %lf, max: %lf\n",bcg1_sigma_l_min,bcg1_sigma_l_max);
	fscanf(fin,"%lf,%lf",&bcg2_sigma_l_min,&bcg2_sigma_l_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bcg2_sigma_l min: %lf, max: %lf\n",bcg2_sigma_l_min,bcg2_sigma_l_max);
	fprintf(flog,"histogram bcg2_sigma_l min: %lf, max: %lf\n",bcg2_sigma_l_min,bcg2_sigma_l_max);
	fscanf(fin,"%lf,%lf",&chi2l_1_min,&chi2l_1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram chi2 bcg1_l min: %lf, max: %lf\n",chi2l_1_min,chi2l_1_max);
	fprintf(flog,"histogram chi2 bcg1_l min: %lf, max: %lf\n",chi2l_1_min,chi2l_1_max);
	fscanf(fin,"%lf,%lf",&chi2l_2_min,&chi2l_2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram chi2 bcg2_l min: %lf, max: %lf\n",chi2l_2_min,chi2l_2_max);
	fprintf(flog,"histogram chi2 bcg2_l min: %lf, max: %lf\n",chi2l_2_min,chi2l_2_max);
	fscanf(fin,"%lf,%lf",&bdif1_min,&bdif1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bdif1 min: %lf, max: %lf\n",bdif1_min,bdif1_max);
	fprintf(flog,"histogram bdif1 min: %lf, max: %lf\n",bdif1_min,bdif1_max);
	fscanf(fin,"%lf,%lf",&bdif2_min,&bdif2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram bdif2 min: %lf, max: %lf\n",bdif2_min,bdif2_max);
	fprintf(flog,"histogram bdif2 min: %lf, max: %lf\n",bdif2_min,bdif2_max);
	fscanf(fin,"%lf,%lf",&hwmax1_min,&hwmax1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram wmax1 min: %lf, max: %lf\n",hwmax1_min,hwmax1_max);
	fprintf(flog,"histogram wmax1 min: %lf, max: %lf\n",hwmax1_min,hwmax1_max);
	fscanf(fin,"%lf,%lf",&hwmax2_min,&hwmax2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram wmax2 min: %lf, max: %lf\n",hwmax2_min,hwmax2_max);
	fprintf(flog,"histogram wmax2 min: %lf, max: %lf\n",hwmax2_min,hwmax2_max);
	fscanf(fin,"%lf,%lf",&hch_wmax1_min,&hch_wmax1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram ch_wmax1 min: %lf, max: %lf\n",hch_wmax1_min,hch_wmax1_max);
	fprintf(flog,"histogram ch_wmax1 min: %lf, max: %lf\n",hch_wmax1_min,hch_wmax1_max);
	fscanf(fin,"%lf,%lf",&hch_wmax2_min,&hch_wmax2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram ch_wmax2 min: %lf, max: %lf\n",hch_wmax2_min,hch_wmax2_max);
	fprintf(flog,"histogram ch_wmax2 min: %lf, max: %lf\n",hch_wmax2_min,hch_wmax2_max);
	fscanf(fin,"%lf,%lf",&chi2sq1_min,&chi2sq1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram chi2sq1 min: %lf, max: %lf\n",chi2sq1_min,chi2sq1_max);
	fprintf(flog,"histogram chi2sq1 min: %lf, max: %lf\n",chi2sq1_min,chi2sq1_max);
	fscanf(fin,"%lf,%lf",&chi2sq2_min,&chi2sq2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram chi2sq2 min: %lf, max: %lf\n",chi2sq2_min,chi2sq2_max);
	fprintf(flog,"histogram chi2sq2 min: %lf, max: %lf\n",chi2sq2_min,chi2sq2_max);
	fscanf(fin,"%lf,%lf",&hch_darea1_min,&hch_darea1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram ch_darea1 min: %lf, max: %lf\n",hch_darea1_min,hch_darea1_max);
	fprintf(flog,"histogram ch_darea1 min: %lf, max: %lf\n",hch_darea1_min,hch_darea1_max);
	fscanf(fin,"%lf,%lf",&hch_darea2_min,&hch_darea2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("histogram ch_darea2 min: %lf, max: %lf\n",hch_darea2_min,hch_darea2_max);
	fprintf(flog,"histogram ch_darea2 min: %lf, max: %lf\n",hch_darea2_min,hch_darea2_max);
	fscanf(fin,"%lf,%lf",&peak1m_min,&peak1m_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("peak1m histogram min: %lf  max: %lf\n",peak1m_min,peak1m_max);
	fprintf(flog,"peak1m histogram min: %lf  max: %lf\n",peak1m_min,peak1m_max);
	fscanf(fin,"%lf,%lf",&peak2m_min,&peak2m_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("peak2m histogram min: %lf  max: %lf\n",peak2m_min,peak2m_max);
	fprintf(flog,"peak2m histogram min: %lf  max: %lf\n",peak2m_min,peak2m_max);
	fscanf(fin,"%lf,%lf",&t1_min,&t1_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("t1 histogram min: %lf, max: %lf\n",t1_min,t1_max);
	fprintf(flog,"t1 histogram min: %lf, max: %lf\n",t1_min,t1_max);
	fscanf(fin,"%lf,%lf",&t2_min,&t2_max);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("t2 histogram min: %lf, max: %lf\n",t2_min,t2_max);
	fprintf(flog,"t2 histogram min: %lf, max: %lf\n",t2_min,t2_max);
	fscanf(fin,"%lf,%lf",&dt_min,&dt_max);
	if(text_mode==1) printf("dt histogram min: %lf, max: %lf\n",dt_min,dt_max);
	fprintf(flog,"dt histogram min: %lf, max: %lf\n",dt_min,dt_max);
	fclose(fin);
	
	//calibration
	//reading file calibration.ini
	if(fexist("calibration.ini")) fin=fopen("calibration.ini","r");
	else exit(0);
	printf("\n");
	printf("Calibration:\n");
	fprintf(flog,"\n");
	fprintf(flog,"Calibration:\n");
	fscanf(fin,"%lf,%lf",&bdif1_peak1_min,&bdif1_peak1_max);
	while((c=getc(fin))!='\n');
	printf("bdif1 peak1 raw range: %lf, %lf\n",bdif1_peak1_min,bdif1_peak1_max);
	fprintf(flog,"bdif1 peak1 raw range: %lf, %lf\n",bdif1_peak1_min,bdif1_peak1_max);
	fscanf(fin,"%lf,%lf",&bdif2_peak1_min,&bdif2_peak1_max);
	while((c=getc(fin))!='\n');
	printf("bdif2 peak1 raw range: %lf, %lf\n",bdif2_peak1_min,bdif2_peak1_max);
	fprintf(flog,"bdif2 peak1 raw range: %lf, %lf\n",bdif2_peak1_min,bdif2_peak1_max);
	fscanf(fin,"%lf,%lf",&bdif1_peak2_min,&bdif1_peak2_max);
	while((c=getc(fin))!='\n');
	printf("bdif1 peak2 raw range: %lf, %lf\n",bdif1_peak2_min,bdif1_peak2_max);
	fprintf(flog,"bdif1 peak2 raw range: %lf, %lf\n",bdif1_peak2_min,bdif1_peak2_max);
	fscanf(fin,"%lf,%lf",&bdif2_peak2_min,&bdif2_peak2_max);
	while((c=getc(fin))!='\n');
	printf("bdif2 peak2 raw range: %lf, %lf\n",bdif2_peak2_min,bdif2_peak2_max);
	fprintf(flog,"bdif2 peak2 raw range: %lf, %lf\n",bdif2_peak2_min,bdif2_peak2_max);
	fscanf(fin,"%lf,%lf",&cal_energy1,&cal_energy2);
	printf("energies of peaks 1,2: %lf, %lf\n",cal_energy1,cal_energy2);
	fprintf(flog,"energies of peaks 1,2: %lf, %lf\n",cal_energy1,cal_energy2);
	fclose(fin);
	
	//watchdogs
	//reading file watchdogs.ini
	if(fexist("watchdogs.ini")) fin=fopen("watchdogs.ini","r");
	else exit(0);
	printf("\n");
	printf("Watchdogs:\n");
	fprintf(flog,"\n");
	fprintf(flog,"Watchdogs:\n");
	fscanf(fin,"%d",&iwmax_wdog);
	while((c=getc(fin))!='\n');
	if(iwmax_wdog==1) printf("wf maximum watchdog: %d\n",iwmax_wdog);
	fprintf(flog,"wf maximum watchdog: %d\n",iwmax_wdog);
	fscanf(fin,"%lf,%lf",&wmax1_l,&wmax1_r);
	while((c=getc(fin))!='\n');
	if(iwmax_wdog==1) printf("wf maximum limits ch1: %lf, %lf\n",wmax1_l,wmax1_r);
	if(iwmax_wdog==1) fprintf(flog,"wf maximum limits ch1: %lf, %lf\n",wmax1_l,wmax1_r);
	fscanf(fin,"%lf,%lf",&wmax2_l,&wmax2_r);
	while((c=getc(fin))!='\n');
	if(iwmax_wdog==1) printf("wf maximum limits ch2: %lf, %lf\n",wmax2_l,wmax2_r);
	if(iwmax_wdog==1) fprintf(flog,"wf maximum limits ch2: %lf, %lf\n",wmax2_l,wmax2_r);
	fscanf(fin,"%d",&ich_wmax_wdog);
	while((c=getc(fin))!='\n');
	if(ich_wmax_wdog==1) printf("wf ch_max watchdog: %d\n",ich_wmax_wdog);
	fprintf(flog,"wf ch_max watchdog: %d\n",ich_wmax_wdog);
	fscanf(fin,"%d,%d",&ch_wmax1_l,&ch_wmax1_r);
	while((c=getc(fin))!='\n');
	if(ich_wmax_wdog==1) printf("wf ch_max1 limits: %d, %d\n",ch_wmax1_l,ch_wmax1_r);
	if(ich_wmax_wdog==1) fprintf(flog,"wf ch_max1 limits: %d, %d\n",ch_wmax1_l,ch_wmax1_r);
	fscanf(fin,"%d,%d",&ch_wmax2_l,&ch_wmax2_r);
	while((c=getc(fin))!='\n');
	if(ich_wmax_wdog==1) printf("wf ch_max2 limits: %d, %d\n",ch_wmax2_l,ch_wmax2_r);
	if(ich_wmax_wdog==1) fprintf(flog,"wf ch_max2 limits: %d, %d\n",ch_wmax2_l,ch_wmax2_r);
	fscanf(fin,"%d",&ichi2sq_wdog);
	while((c=getc(fin))!='\n');
	if(ichi2sq_wdog==1) printf("parabolic fit watchdog: %d\n",ichi2sq_wdog);
	fprintf(flog,"parabolic fit watchdog: %d\n",ichi2sq_wdog);
	fscanf(fin,"%lf,%lf",&chi2sq1_l,&chi2sq1_r);
	while((c=getc(fin))!='\n');
	if(ichi2sq_wdog==1) printf("chi2sq1  limits: %lf, %lf\n",chi2sq1_l,chi2sq1_r);
	if(ichi2sq_wdog==1) fprintf(flog,"chi2sq1  limits: %lf, %lf\n",chi2sq1_l,chi2sq1_r);
	fscanf(fin,"%lf,%lf",&chi2sq2_l,&chi2sq2_r);
	while((c=getc(fin))!='\n');
	if(ichi2sq_wdog==1) printf("chi2sq2  limits: %lf, %lf\n",chi2sq2_l,chi2sq2_r);
	if(ichi2sq_wdog==1) fprintf(flog,"chi2sq2  limits: %lf, %lf\n",chi2sq2_l,chi2sq2_r);
	fscanf(fin,"%d",&ibcg_sigma_l_wdog);
	while((c=getc(fin))!='\n');
	if(ibcg_sigma_l_wdog==1) printf("rms of baseline watchdog: %d\n",ibcg_sigma_l_wdog);
	fprintf(flog,"rms of baseline watchdog: %d\n",ibcg_sigma_l_wdog);
	fscanf(fin,"%lf,%lf",&bcg1_sigma_l_l_limit,&bcg1_sigma_l_r_limit);
	while((c=getc(fin))!='\n');
	if(ibcg_sigma_l_wdog==1) printf("bcg sigma 1 limits: %lf, %lf\n",bcg1_sigma_l_l_limit,bcg1_sigma_l_r_limit);
	if(ibcg_sigma_l_wdog==1) fprintf(flog,"bcg sigma 1 limits: %lf, %lf\n",bcg1_sigma_l_l_limit,bcg1_sigma_l_r_limit);
	fscanf(fin,"%lf,%lf",&bcg2_sigma_l_l_limit,&bcg2_sigma_l_r_limit);
	if(ibcg_sigma_l_wdog==1) printf("bcg sigma 2 limits: %lf, %lf\n",bcg2_sigma_l_l_limit,bcg2_sigma_l_r_limit);
	if(ibcg_sigma_l_wdog==1) fprintf(flog,"bcg sigma 2 limits: %lf, %lf\n",bcg2_sigma_l_l_limit,bcg2_sigma_l_r_limit);
	fclose(fin);
	
	
	//suffixes
	//reading file suffix.ini
	if(fexist("suffix.ini")) fin=fopen("suffix.ini","r");
	else exit(0);
	if(text_mode==1) printf("\n");
	if(text_mode==1) printf("suffixes:\n");
	fprintf(flog,"\n");
	fprintf(flog,"suffixes:\n");
	fscanf(fin,"%s",&suffix_m1);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode -1, suffix %s\n",suffix_m1);
	fprintf(flog,"mode -1, suffix %s\n",suffix_m1);
	fscanf(fin,"%s",&suffix_0);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode 0, suffix %s\n",suffix_0);
	fprintf(flog,"mode 0, suffix %s\n",suffix_0);
	fscanf(fin,"%s",&suffix_1);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode 1, suffix %s\n",suffix_1);
	fprintf(flog,"mode 1, suffix %s\n",suffix_1);
	fscanf(fin,"%s",&suffix_2);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode 2, suffix %s\n",suffix_2);
	fprintf(flog,"mode 2, suffix %s\n",suffix_2);
	fscanf(fin,"%s",&suffix_3);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode 3, suffix %s\n",suffix_3);
	fprintf(flog,"mode 3, suffix %s\n",suffix_3);
	fscanf(fin,"%s",&suffix_4);
	while((c=getc(fin))!='\n');
	if(text_mode==1) printf("mode 4, suffix %s\n",suffix_4);
	fprintf(flog,"mode 4, suffix %s\n",suffix_4);
	fscanf(fin,"%s",&suffix_5);
	if(text_mode==1) printf("mode 5, suffix %s\n",suffix_5);
	fprintf(flog,"mode 5, suffix %s\n",suffix_5);
	fclose(fin);
	printf("\n");
	fprintf(flog,"\n");

	
	if(i_mode==-1) 
	{
			fwav1=fopen("wav1.txt","w");	
			fwav2=fopen("wav2.txt","w");
			fwdif1=fopen("wdif1.txt","w");
			fwdif2=fopen("wdif2.txt","w");
			fparabola1=fopen("parabola1.txt","w");
			fparabola2=fopen("parabola2.txt","w");
	}
	if(i_mode==2) febin=fopen("energies-2.dat","wb+");
	if(i_mode==3) febin=fopen("energies-3.dat","wb+");
	if(i_mode==4) febin=fopen("energies-4.dat","wb+");
	if(i_mode==5) febin=fopen("energies-5.dat","wb+");
	
		
	if(i_mode==0) fcal=fopen("e-cal.txt","w");
	//reading of calibration 
	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5)
	{ 
			//creation of average calibration
			printf("opening e-cal.txt calibration file....");
			fprintf(flog,"opening e-cal.txt calibration file....");
			if(fexist("e-cal.txt")==1) fcal=fopen("e-cal.txt","r");
			else exit(0);
			fcal1=fopen("e1-cal.txt","w");
			bdif1_peak1_avg=0;
			bdif1_peak2_avg=0;
			bdif2_peak1_avg=0;
			bdif2_peak2_avg=0;
			i=0;
			for(l=1;l<=n_sess;l++)
			{
				fscanf(fcal,"%ld %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",&ipom,&bcg1_sess[l],&bdif1_peak1,&w1_peak1,&bdif1_peak2,&w1_peak2,&bcg2_sess[l],&bdif2_peak1,&w2_peak1,&bdif2_peak2,&w2_peak2);
				bdif1_peak1_avg=bdif1_peak1_avg+bdif1_peak1/(double)n_sess;
				bdif1_peak2_avg=bdif1_peak2_avg+bdif1_peak2/(double)n_sess;
				bdif2_peak1_avg=bdif2_peak1_avg+bdif2_peak1/(double)n_sess;
				bdif2_peak2_avg=bdif2_peak2_avg+bdif2_peak2/(double)n_sess;
			}
			fclose(fcal);
		//prepsani kalibrace
		//bdif1_peak1_avg=35626;
		//bdif2_peak1_avg=35205;
		a1_cal_avg=(cal_energy2-cal_energy1)/(bdif1_peak2_avg-bdif1_peak1_avg);
		a2_cal_avg=(cal_energy2-cal_energy1)/(bdif2_peak2_avg-bdif2_peak1_avg);
		b1_cal_avg=cal_energy1-bdif1_peak1_avg*a1_cal_avg;
		b2_cal_avg=cal_energy1-bdif2_peak1_avg*a2_cal_avg;
		printf("finished O.K.\n");
		fprintf(flog,"finished O.K.\n");
	}

	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
	{
		printf("opening stability file....");
		fprintf(flog,"opening stability file....");
		fstab=fopen("stability.txt","r");
		for(l=1;l<=n_sess;l++)
		{
			fscanf(fstab,"%ld %lf %lf %lf %lf %lf %lf",&ipom,&bcg1_sess[l],&bdif1_sess[l],&fbdif1_sess[l],&bcg2_sess[l],&bdif2_sess[l],&fbdif2_sess[l]);
			
		}
		fclose(fstab);
		printf("finished O.K.\n");
		fprintf(flog,"finished O.K.\n");
	} //if i_mode==1 or 2 or 3
	
	
	//inicializace
	for(k=0;k<maxhist_small;k++)	
	{
		hdarea1[k]=hwdif1_max[k]=hchi2l_1[k]=0;
		hdarea2[k]=hwdif2_max[k]=hchi2l_2[k]=0;
		hbcg1_mean_l[k]=hbcg1_sigma_l[k]=0;
		hbcg2_mean_l[k]=hbcg2_sigma_l[k]=0;
		hbcg1_mean_l_sess[k]=hbcg2_mean_l_sess[k]=0;
		ht1[k]=ht2[k]=hdt[k]=0;
		hchi2sq1[k]=hchi2sq2[k]=0;
		
	}
			
	for(k=0;k<maxhist;k++) hbdif1[k]=hbdif2[k]=hbdif1_cal[k]=hbdif2_cal[k]=hpmax1[k]=hpmax2[k]=hwmax1[k]=hwmax2[k]=haif[k]=0;
	for(k=0;k<maxhist_small2;k++) hch_wmax1[k]=hch_wmax2[k]=hch_darea1[k]=hch_darea2[k]=0;
	for(k=0;k<max_sess;k++) 
	{
		hbdif1_sess[k]=hbdif2_sess[k]=0;
		pp1_mean[k]=pp2_mean[k]=0.0;
	}
	for(k=0;k<maxhist_px;k++) peak1m[k]=peak2m[k]=0;
	for(k=0;k<maxhist_2D;k++) hrf[k]=hdb[k]=0;

	
	//alokace 2D pole
	if(i_mode==0 || i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4)
	{
		pp1=(int(*)[maxhist_py])malloc(maxhist_px*maxhist_py*sizeof(int));
		pp2=(int(*)[maxhist_py])malloc(maxhist_px*maxhist_py*sizeof(int));
		
		for(k=0;k<maxhist_px;k++)
		{
			for(l=0;l<maxhist_py;l++) 
			{
				pp1[k][l]=pp2[k][l]=0;
				peak1m[k]=peak2m[k]=0;
			}
		}
		if(pp1==NULL)
		{
			printf("Not enough memory to allocate pp1\n");
			exit(0);
		}
		if(pp2==NULL)
		{
			printf("Not enough memory to allocate pp2\n");
			exit(0);
		}
		
	}//if i_mode==0,1,2,3,4
	
	
	if(i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
	{
		ee=(int(*)[maxhist_2D])malloc(maxhist_2D*maxhist_2D*sizeof(int));
		for(k=0;k<maxhist_2D;k++)
		{
			for(l=0;l<maxhist_2D;l++) ee[k][l]=0;
		}
		
		
	}//if i_mode==2,3,4,5

	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5)
		{
			printf("opening peak shape file....");
			fprintf(flog,"opening peak shape file....");
			fpeakm=fopen("peakm.txt","r");
			for(i=0;i<maxhist_px;i++)	
			{
				fscanf(fpeakm,"%ld %lf %lf %lf %lf %lf %lf",&ipom,&peak1m[i],&peak1l[i],&peak1r[i],&peak2m[i],&peak2l[i],&peak2r[i]);
			}
			fclose(fpeakm);
			printf("finished O.K.\n");
			fprintf(flog,"finished O.K.\n");
		} //if i_mode==1,2,3,4,5

	

	n_all=n_good=n_good1=n_good2=0;
	pp1_counter=pp2_counter=0;
	bdif1_sess_avg=bdif2_sess_avg=0.0;

	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
		{
			if(fexist("e-cal.txt")==1) fcal=fopen("e-cal.txt","r");	
			else exit(0);
	}
	
	
	

	//main loop
	for(l=1;l<=n_sess;l++)
	{
		
		if(i_mode==0 || i_mode==1) //nulovani histogramu pro session
		{
			for(k=0;k<maxhist_small;k++) hbcg1_mean_l_sess[k]=hbdif1_sess[k]=hbcg2_mean_l_sess[k]=hbdif2_sess[k]=0;
			for(k=0;k<maxhist;k++) hbdif1_sess[k]=hbdif2_sess[k]=0;
		}

		//cteni z kalibracniho souboru e-cal.txt
		if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
		{
			
			fscanf(fcal,"%ld  %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",&ipom,&bcg1_sess[l],&bdif1_peak1,&w1_peak1,&bdif1_peak2,&w1_peak2,&bcg2_sess[l],&bdif2_peak1,&w2_peak1,&bdif2_peak2,&w2_peak2);			
			a1_cal=(cal_energy2-cal_energy1)/(bdif1_peak2-bdif1_peak1);
			a2_cal=(cal_energy2-cal_energy1)/(bdif2_peak2-bdif2_peak1);
			b1_cal=cal_energy1-bdif1_peak1*a1_cal;
			b2_cal=cal_energy1-bdif2_peak1*a2_cal;
			printf("a1=%18.6lf b1=%18.6lf\n",a1_cal,b1_cal); 
			printf("a2=%18.6lf b2=%18.6lf\n",a2_cal,b2_cal); 
		}
		
		//otvirani spektra
		if(i_mode==-1) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_m1);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==0) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_0);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==1) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_1);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==2) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_2);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==3) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_3);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==4) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_4);
			fsp=fopen(sp_file,"rb");
		}
		if(i_mode==5) 
		{
			sprintf(sp_file,"%s-%d.%s",sp_path,l,suffix_5);
			fsp=fopen(sp_file,"rb");
		}
		
		
		//reading header
		fread(&r_nbrSamples,sizeof(r_nbrSamples),1,fsp);
		fread(&r_nbrSegments,sizeof(r_nbrSegments),1,fsp);
		fread(&r_nbrWaveforms,sizeof(r_nbrWaveforms),1,fsp);
		printf("session %d file %s\n",l,sp_file);
		
		//reading waveforms
		for(j=0;j<n_wav;j++)
		{
			i_good1=1;
			i_good2=1;
			
			n_all++;
			wdif1[0]=wdif2[0]=0;
			wdif1_max=wdif2_max=0;
			
			//reading
			for(i=0;i<n_points;i++) 
			{
				fread(&wav1[i],sizeof(wav1[i]),1,fsp);
				fread(&wav2[i],sizeof(wav2[i]),1,fsp);
				wav1[i]=wav1[i]/raw_divider;
				wav2[i]=wav2[i]/raw_divider;
				if(i_mode==-1)
				{
					fprintf(fwav1,"%6ld %8d \n",i,wav1[i]);
					fprintf(fwav2,"%6ld %8d \n",i,wav2[i]);
				}
				
				wdif1[i]=0;
				wdif2[i]=0;
			
			}//i
			
			get_wdif(wdif1,wav1,10,n_points);
			get_wdif(wdif2,wav2,10,n_points);
			smooth(wdif1,wdif1_smooth,n_points);
			smooth(wdif2,wdif2_smooth,n_points);
			if(i_mode==-1)
			{
				for(i=0;i<n_points;i++) 
				{
					fprintf(fwdif1,"%6ld %8d \n",i,wdif1[i]);
					fprintf(fwdif2,"%6ld %8d \n",i,wdif2[i]);
				}
				fprintf(fwav1,"\n");
				fprintf(fwav2,"\n");
				fprintf(fwdif1,"\n");
				fprintf(fwdif2,"\n");
			}
					
		//wdif_max
		get_imax2(&wdif1_max,&ch_wdif1_max,&chi2_pom,wdif1,0,n_points-1,20);
		get_imax2(&wdif2_max,&ch_wdif2_max,&chi2_pom,wdif2,0,n_points-1,20);
						
		//darea
		get_iarea(&darea1,&ch_darea1_l,&ch_darea1_r,(int)ch_wdif1_max,wdif1,wdif1_treshold);
		get_iarea(&darea2,&ch_darea2_l,&ch_darea2_r,(int)ch_wdif2_max,wdif2,wdif2_treshold);
		ch_darea1=ch_darea1_r-ch_darea1_l;
		ch_darea2=ch_darea2_r-ch_darea2_l;
				
		//bcg		
		get_bcg(&bcg1_mean_l,&bcg1_sigma_l,wav1,0,ch_darea1_l);
		get_bcg(&bcg2_mean_l,&bcg2_sigma_l,wav2,0,ch_darea2_l);
		
		
		if(i_mode==-1) 
		{
				get_simax2p(&wmax1,&fch_wmax1,&chi2sq1,&ich_wmax1,&c2,&c1,&c0,wav1,0,n_points-1,wmax_fit_range);
				for(k=ich_wmax1-20;k<=ich_wmax1+20;k++)	fprintf(fparabola1,"%d %10.6lf\n",k,c2*pow((double)k,2)+c1*(double)k+c0);
				fprintf(fparabola1,"\n");
				get_simax2p(&wmax2,&fch_wmax2,&chi2sq2,&ich_wmax2,&c2,&c1,&c0,wav2,0,n_points-1,wmax_fit_range);
				for(k=ich_wmax2-20;k<=ich_wmax2+20;k++)	fprintf(fparabola2,"%d %10.6lf\n",k,c2*pow((double)k,2)+c1*(double)k+c0);
				fprintf(fparabola2,"\n");
		}
		else 
		{
				get_simax2(&wmax1,&fch_wmax1,&chi2sq1,wav1,0,n_points-1,wmax_fit_range);
				get_simax2(&wmax2,&fch_wmax2,&chi2sq2,wav2,0,n_points-1,wmax_fit_range);
		}
		ch_wmax1=mround(fch_wmax1);
		ch_wmax2=mround(fch_wmax2);
		bdif1=wmax1-bcg1_mean_l;
		bdif2=wmax2-bcg2_mean_l;
	
		get_chi2(&chi2l_1,bcg1_mean_l,wav1,0,ch_darea1_l);
		chi2l_1=chi2l_1/pow(bcg1_sigma_l,2);
		get_chi2(&chi2l_2,bcg2_mean_l,wav2,0,ch_darea2_l);
		chi2l_2=chi2l_2/pow(bcg2_sigma_l,2);
								
		//fixed watchdogs
		if(iwmax_wdog==1) //wmax - waveform maximum
		{
				if(wmax1<wmax1_l || wmax1>wmax1_r) i_good1=0;
				if(wmax2<wmax2_l || wmax2>wmax2_r) i_good2=0;
		}
		
		if(ich_wmax_wdog==1) //position of waveform maximum 
		{
			if(ch_wmax1<=ch_wmax1_l || ch_wmax1>=ch_wmax1_r) i_good1=0;
			if(ch_wmax2<=ch_wmax2_l || ch_wmax2>=ch_wmax2_r) i_good2=0;
		}
		
		if(ichi2sq_wdog==1) //chi2sq - chi2 parabolickeho fitu maxima
		{
			if(chi2sq1<chi2sq1_l || chi2sq1>chi2sq1_r) i_good1=0;
			if(chi2sq2<chi2sq2_l || chi2sq2>chi2sq2_r) i_good2=0;
		}
		
		if(ibcg_sigma_l_wdog==1) //bcg_sigma_l - rms of baseline
		{
			if(bcg1_sigma_l<bcg1_sigma_l_l_limit || bcg1_sigma_l>bcg1_sigma_l_r_limit) i_good1=0;
			if(bcg2_sigma_l<bcg2_sigma_l_l_limit || bcg2_sigma_l>bcg2_sigma_l_r_limit) i_good2=0;
		}
			
				
		if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5)
		{
			if(avg_cal==1)
			{
				bdif1_cal=bdif1*a1_cal_avg+b1_cal_avg;
				bdif2_cal=bdif2*a2_cal_avg+b2_cal_avg;
			}
			else
			{
				bdif1_cal=bdif1*a1_cal+b1_cal;
				bdif2_cal=bdif2*a2_cal+b2_cal;
			}

			//bcg1_mean_l_l_limit=bcg1_sess[l]+bcg1_mean_l_l_limit_rel;
			//bcg2_mean_l_l_limit=bcg2_sess[l]+bcg2_mean_l_l_limit_rel;
		}
				
		
			//timing
			//get_t(&t1,frac1,wav1,ch_wmax1,ch_darea1_l);
			//get_t(&t2,frac2,wav2,ch_wmax2,ch_darea2_l);
			t1=fch_wmax1;
			t2=fch_wmax2;
			shift1=ref_point1-(int)t1;
			shift2=ref_point2-(int)t2;
			dt=t2-t1;
			if(i_good1==1 && fabs((double)shift1)>shift_offset)
			{
				printf("Warring channel 1: shift %d > shift_offset %d\n",shift1,shift_offset);
				fprintf(flog,"Warring channel 1: shift %d > shift_offset %d\n",shift1,shift_offset);
				printf("\n");
				i_good1=0;
			}
			if(i_good2==1 && fabs((double)shift2)>shift_offset)
			{
				printf("Warring channel 2: shift %d > shift_offset %d\n",shift2,shift_offset);
				fprintf(flog,"Warring channel 2: shift %d > shift_offset %d\n",shift2,shift_offset);
				printf("\n");
				i_good2=0;
			}		
			
					
						
			//histogramy
			if(i_mode==0)
			{
				if(i_good1==1)
				{
					
					
					for(i=0;i<n_points;i++)
					{
						if(i>=shift_offset && i<n_points-shift_offset)
						{
							wav1_n[i]=(double)(wav1[i-shift1]-bcg1_mean_l)/bdif1*1000;
						}
						else wav1_n[i]=0;
									
							add_hist(wav1_n[i],pp1[i],peak1m_min,peak1m_max,maxhist_py);
							pp1_mean[i]=pp1_mean[i]+wav1_n[i];
														
					}
					
					pp1_counter++;

					add_hist(darea1,hdarea1,darea1_min,darea1_max,maxhist_small);
					add_hist(ch_wdif1_max,hwdif1_max,wdif1_max_min,wdif1_max_max,maxhist_small);
					add_hist(wmax1,hwmax1,hwmax1_min,hwmax1_max,maxhist);
					add_hist(ch_wmax1,hch_wmax1,hch_wmax1_min,hch_wmax1_max,maxhist_small2);
					add_hist(ch_darea1,hch_darea1,hch_darea1_min,hch_darea1_max,maxhist_small2);
				
					add_hist(bcg1_mean_l,hbcg1_mean_l,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
					add_hist(bcg1_mean_l,hbcg1_mean_l_sess,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
					add_hist(bcg1_sigma_l,hbcg1_sigma_l,bcg1_sigma_l_min,bcg1_sigma_l_max,maxhist_small);
					add_hist(chi2l_1,hchi2l_1,chi2l_1_min,chi2l_1_max,maxhist_small);
																									
					add_hist(bdif1,hbdif1,bdif1_min,bdif1_max,maxhist);
					add_hist(chi2sq1,hchi2sq1,chi2sq1_min,chi2sq1_max,maxhist_small);
					add_hist(bdif1,hbdif1_sess,bdif1_min,bdif1_max,maxhist);
					
				}//if i_good==1

				if(i_good2==1)
				{
					
			
						
					for(i=0;i<n_points;i++)
					{
						if(i>=shift_offset && i<n_points-shift_offset)
						{
							wav2_n[i]=(double)(wav2[i-shift2]-bcg2_mean_l)/bdif2*1000;
						}
						else wav2_n[i]=0;
						add_hist(wav2_n[i],pp2[i],peak2m_min,peak2m_max,maxhist_py);
						pp2_mean[i]=pp2_mean[i]+wav2_n[i];
					
					}
					pp2_counter++;

					add_hist(darea2,hdarea2,darea2_min,darea2_max,maxhist_small);
					add_hist(ch_wdif2_max,hwdif2_max,wdif2_max_min,wdif2_max_max,maxhist_small);
					add_hist(wmax2,hwmax2,hwmax2_min,hwmax2_max,maxhist);
					add_hist(ch_wmax2,hch_wmax2,hch_wmax2_min,hch_wmax2_max,maxhist_small2);
					add_hist(ch_darea2,hch_darea2,hch_darea2_min,hch_darea2_max,maxhist_small2);
				
					add_hist(bcg2_mean_l,hbcg2_mean_l,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
					add_hist(bcg2_mean_l,hbcg2_mean_l_sess,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
					add_hist(bcg2_sigma_l,hbcg2_sigma_l,bcg2_sigma_l_min,bcg2_sigma_l_max,maxhist_small);
					add_hist(chi2l_2,hchi2l_2,chi2l_2_min,chi2l_2_max,maxhist_small);
															
					add_hist(bdif2,hbdif2,bdif2_min,bdif2_max,maxhist);
					add_hist(chi2sq2,hchi2sq2,chi2sq2_min,chi2sq2_max,maxhist_small);
					add_hist(bdif2,hbdif2_sess,bdif2_min,bdif2_max,maxhist);

					
				}//if i_good2==1
			}//if i_mode==0
		
			if(i_mode==1)
			{
				
				for(i=0;i<n_points;i++)
					{
						if(i>=shift_offset && i<n_points-shift_offset)
						{
							wav1_n[i]=(double)(wav1[i-shift1]-bcg1_mean_l)/bdif1*1000;
							wav2_n[i]=(double)(wav2[i-shift2]-bcg2_mean_l)/bdif2*1000;
						}
						else 
						{
							wav1_n[i]=0;	
							wav2_n[i]=0;	
						}
					
					}
				
					//	dsmooth(wav1_n,20,maxhist_px);
					//	dsmooth(wav2_n,20,maxhist_px);
					for(i=shift_offset;i<n_points-shift_offset;i++)
					{
						if(wav1_n[i]<peak1m[i]-f_ext_l*(peak1m[i]-peak1l[i]) || wav1_n[i]>peak1m[i]+f_ext_l*(peak1r[i]-peak1m[i])) i_good1=0;
						if(wav2_n[i]<peak2m[i]-f_ext_l*(peak2m[i]-peak2l[i]) || wav2_n[i]>peak2m[i]+f_ext_l*(peak2r[i]-peak2m[i])) i_good2=0;
					}
				
				if(i_good1==1)
				{
					for(i=0;i<n_points;i++)
					{
						add_hist(wav1_n[i],pp1[i],peak1m_min,peak1m_max,maxhist_py);
						pp1_mean[i]=pp1_mean[i]+wav1_n[i];
									
					}
					pp1_counter++;

					add_hist(darea1,hdarea1,darea1_min,darea1_max,maxhist_small);
					add_hist(ch_wdif1_max,hwdif1_max,wdif1_max_min,wdif1_max_max,maxhist_small);
					add_hist(wmax1,hwmax1,hwmax1_min,hwmax1_max,maxhist);
					add_hist(ch_wmax1,hch_wmax1,hch_wmax1_min,hch_wmax1_max,maxhist_small2);
					add_hist(ch_darea1,hch_darea1,hch_darea1_min,hch_darea1_max,maxhist_small2);
				
					add_hist(bcg1_mean_l,hbcg1_mean_l,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
					add_hist(bcg1_mean_l,hbcg1_mean_l_sess,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
					add_hist(bcg1_sigma_l,hbcg1_sigma_l,bcg1_sigma_l_min,bcg1_sigma_l_max,maxhist_small);
					add_hist(chi2l_1,hchi2l_1,chi2l_1_min,chi2l_1_max,maxhist_small);
																									
					add_hist(bdif1,hbdif1,bdif1_min,bdif1_max,maxhist);
					add_hist(chi2sq1,hchi2sq1,chi2sq1_min,chi2sq1_max,maxhist_small);
					add_hist(bdif1,hbdif1_sess,bdif1_min,bdif1_max,maxhist);
					add_hist(bdif1_cal,hbdif1_cal,bdif1_cal_min,bdif1_cal_max,maxhist); //calibrated
					
				
				}//if i_good==1

				if(i_good2==1)
				{
					for(i=0;i<n_points;i++)
					{
						add_hist(wav2_n[i],pp2[i],peak2m_min,peak2m_max,maxhist_py);
						pp2_mean[i]=pp2_mean[i]+wav2_n[i];
					
					}
					pp2_counter++;

					add_hist(darea2,hdarea2,darea2_min,darea2_max,maxhist_small);
					add_hist(ch_wdif2_max,hwdif2_max,wdif2_max_min,wdif2_max_max,maxhist_small);
					add_hist(wmax2,hwmax2,hwmax2_min,hwmax2_max,maxhist);
					add_hist(ch_wmax2,hch_wmax2,hch_wmax2_min,hch_wmax2_max,maxhist_small2);
					add_hist(ch_darea2,hch_darea2,hch_darea2_min,hch_darea2_max,maxhist_small2);
				
					add_hist(bcg2_mean_l,hbcg2_mean_l,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
					add_hist(bcg2_mean_l,hbcg2_mean_l_sess,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
					add_hist(bcg2_sigma_l,hbcg2_sigma_l,bcg2_sigma_l_min,bcg2_sigma_l_max,maxhist_small);
					add_hist(chi2l_2,hchi2l_2,chi2l_2_min,chi2l_2_max,maxhist_small);
															
					add_hist(bdif2,hbdif2,bdif2_min,bdif2_max,maxhist);
					add_hist(chi2sq2,hchi2sq2,chi2sq2_min,chi2sq2_max,maxhist_small);
					add_hist(bdif2,hbdif2_sess,bdif2_min,bdif2_max,maxhist);
					add_hist(bdif2_cal,hbdif2_cal,bdif2_cal_min,bdif2_cal_max,maxhist); //calibrated
					
				}//if i_good==1
			}//if i_mode==1
			

			if(i_mode==2 || i_mode==3)
			{
			if(i_good1==1 && i_good2==1)
			{
				for(i=0;i<n_points;i++)
				{
					if(i>=shift_offset && i<n_points-shift_offset)
						{
							wav1_n[i]=(double)(wav1[i-shift1]-bcg1_mean_l)/bdif1*1000;
							wav2_n[i]=(double)(wav2[i-shift2]-bcg2_mean_l)/bdif2*1000;
						}
						else 
						{
							wav1_n[i]=0;	
							wav2_n[i]=0;	
						}
												
						add_hist(wav1_n[i],pp1[i],peak1m_min,peak1m_max,maxhist_py);
						add_hist(wav2_n[i],pp2[i],peak2m_min,peak2m_max,maxhist_py);
						
						pp1_mean[i]=pp1_mean[i]+wav1_n[i];
						pp2_mean[i]=pp2_mean[i]+wav2_n[i];
						
				}
				pp1_counter++;
				pp2_counter++;

				add_hist(darea1,hdarea1,darea1_min,darea1_max,maxhist_small);
				add_hist(darea2,hdarea2,darea2_min,darea2_max,maxhist_small);
				add_hist(ch_wdif1_max,hwdif1_max,wdif1_max_min,wdif1_max_max,maxhist_small);
				add_hist(ch_wdif2_max,hwdif2_max,wdif2_max_min,wdif2_max_max,maxhist_small);
				
				add_hist(wmax1,hwmax1,hwmax1_min,hwmax1_max,maxhist);
				add_hist(wmax2,hwmax2,hwmax2_min,hwmax2_max,maxhist);
				add_hist(ch_wmax1,hch_wmax1,hch_wmax1_min,hch_wmax1_max,maxhist_small2);
				add_hist(ch_wmax2,hch_wmax2,hch_wmax2_min,hch_wmax2_max,maxhist_small2);
				add_hist(ch_darea1,hch_darea1,hch_darea1_min,hch_darea1_max,maxhist_small2);
				add_hist(ch_darea2,hch_darea2,hch_darea2_min,hch_darea2_max,maxhist_small2);

				add_hist(bcg1_mean_l,hbcg1_mean_l,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
				add_hist(bcg2_mean_l,hbcg2_mean_l,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
				add_hist(bcg1_mean_l,hbcg1_mean_l_sess,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
				add_hist(bcg2_mean_l,hbcg2_mean_l_sess,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
				add_hist(bcg1_sigma_l,hbcg1_sigma_l,bcg1_sigma_l_min,bcg1_sigma_l_max,maxhist_small);
				add_hist(bcg2_sigma_l,hbcg2_sigma_l,bcg2_sigma_l_min,bcg2_sigma_l_max,maxhist_small);
				add_hist(chi2l_1,hchi2l_1,chi2l_1_min,chi2l_1_max,maxhist_small);
				add_hist(chi2l_2,hchi2l_2,chi2l_2_min,chi2l_2_max,maxhist_small);
				
								
				add_hist(bdif1_cal,hbdif1_cal,bdif1_cal_min,bdif1_cal_max,maxhist);
				add_hist(bdif2_cal,hbdif2_cal,bdif2_cal_min,bdif2_cal_max,maxhist);
				add_hist(bdif1,hbdif1,bdif1_min,bdif1_max,maxhist);
				add_hist(bdif2,hbdif2,bdif2_min,bdif2_max,maxhist);
				add_hist(chi2sq1,hchi2sq1,chi2sq1_min,chi2sq1_max,maxhist_small);
				add_hist(chi2sq2,hchi2sq2,chi2sq2_min,chi2sq2_max,maxhist_small);
				add_hist(bdif1,hbdif1_sess,bdif1_min,bdif1_max,maxhist);
				add_hist(bdif2,hbdif2_sess,bdif2_min,bdif2_max,maxhist);
				
				
				//timing
				add_hist(t1,ht1,t1_min,t1_max,maxhist_small);
				add_hist(t2,ht2,t2_min,t2_max,maxhist_small);
				add_hist(dt,hdt,dt_min,dt_max,maxhist_small);
				//cuts
				rf=bdif1_cal+bdif2_cal-2*511.0;
				db=bdif1_cal-bdif2_cal;
				//printf("%lf %lf %lf %lf\n",bdif1_cal,bdif2_cal,rf,db);
				if(fabs(db)<=rf_delta) add_hist(rf,hrf,hrf_min,hrf_max,maxhist_2D);
				if(fabs(rf)<=db_delta) add_hist(db,hdb,hdb_min,hdb_max,maxhist_2D);
				//2D hist
				add_2Dhist(db,rf,ee,hrf_min,hrf_max,maxhist_2D);
				//aif 
				if(fabs(rf-(sqrt(pow(db,2)+pow(511.0,2))-511.0))<d_aif) add_hist(db,haif,aif_min,aif_max,maxhist);

				fwrite(&bdif1_cal,sizeof(bdif1_cal),1,febin);
				fwrite(&bdif2_cal,sizeof(bdif2_cal),1,febin);
				fwrite(&dt,sizeof(dt),1,febin);
				
			}//i_good	
								
			}// if i_mode==2

			if(i_mode==4 || i_mode==5)
			{
				for(i=0;i<n_points;i++)
				{
					
					if(i>=shift_offset && i<n_points-shift_offset)
						{
							wav1_n[i]=(double)(wav1[i-shift1]-bcg1_mean_l)/bdif1*1000;
							wav2_n[i]=(double)(wav2[i-shift2]-bcg2_mean_l)/bdif2*1000;
						}
						else 
						{
							wav1_n[i]=0;	
							wav2_n[i]=0;	
						}

				}
				
				//	dsmooth(wav1_n,20,maxhist_px);
				//	dsmooth(wav2_n,20,maxhist_px);
				for(i=shift_offset;i<n_points-shift_offset;i++)
				{
					if(wav1_n[i]<peak1m[i]-f_ext_l*(peak1m[i]-peak1l[i]) || wav1_n[i]>peak1m[i]+f_ext_l*(peak1r[i]-peak1m[i])) i_good1=0;
					if(wav2_n[i]<peak2m[i]-f_ext_l*(peak2m[i]-peak2l[i]) || wav2_n[i]>peak2m[i]+f_ext_l*(peak2r[i]-peak2m[i])) i_good2=0;
				}
				
					
			if(i_good1==1 && i_good2==1)
			{
				for(i=0;i<n_points;i++)
				{
						add_hist(wav1_n[i],pp1[i],peak1m_min,peak1m_max,maxhist_py);
						add_hist(wav2_n[i],pp2[i],peak2m_min,peak2m_max,maxhist_py);
						pp1_mean[i]=pp1_mean[i]+wav1_n[i];
						pp2_mean[i]=pp2_mean[i]+wav2_n[i];
															
				}	
				pp1_counter++;
				pp2_counter++;
				
				
				add_hist(darea1,hdarea1,darea1_min,darea1_max,maxhist_small);
				add_hist(darea2,hdarea2,darea2_min,darea2_max,maxhist_small);
				add_hist(ch_wdif1_max,hwdif1_max,wdif1_max_min,wdif1_max_max,maxhist_small);
				add_hist(ch_wdif2_max,hwdif2_max,wdif2_max_min,wdif2_max_max,maxhist_small);
				
				add_hist(wmax1,hwmax1,hwmax1_min,hwmax1_max,maxhist);
				add_hist(wmax2,hwmax2,hwmax2_min,hwmax2_max,maxhist);
				add_hist(ch_wmax1,hch_wmax1,hch_wmax1_min,hch_wmax1_max,maxhist_small2);
				add_hist(ch_wmax2,hch_wmax2,hch_wmax2_min,hch_wmax2_max,maxhist_small2);
				add_hist(ch_darea1,hch_darea1,hch_darea1_min,hch_darea1_max,maxhist_small2);
				add_hist(ch_darea2,hch_darea2,hch_darea2_min,hch_darea2_max,maxhist_small2);

				add_hist(bcg1_mean_l,hbcg1_mean_l,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
				add_hist(bcg2_mean_l,hbcg2_mean_l,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
				add_hist(bcg1_mean_l,hbcg1_mean_l_sess,bcg1_mean_l_min,bcg1_mean_l_max,maxhist_small);
				add_hist(bcg2_mean_l,hbcg2_mean_l_sess,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
				add_hist(bcg1_sigma_l,hbcg1_sigma_l,bcg1_sigma_l_min,bcg1_sigma_l_max,maxhist_small);
				add_hist(bcg2_sigma_l,hbcg2_sigma_l,bcg2_sigma_l_min,bcg2_sigma_l_max,maxhist_small);
				add_hist(chi2l_1,hchi2l_1,chi2l_1_min,chi2l_1_max,maxhist_small);
				add_hist(chi2l_2,hchi2l_2,chi2l_2_min,chi2l_2_max,maxhist_small);
				
								
				add_hist(bdif1_cal,hbdif1_cal,bdif1_cal_min,bdif1_cal_max,maxhist);
				add_hist(bdif2_cal,hbdif2_cal,bdif2_cal_min,bdif2_cal_max,maxhist);
				add_hist(bdif1,hbdif1,bdif1_min,bdif1_max,maxhist);
				add_hist(bdif2,hbdif2,bdif2_min,bdif2_max,maxhist);
				add_hist(chi2sq1,hchi2sq1,chi2sq1_min,chi2sq1_max,maxhist_small);
				add_hist(chi2sq2,hchi2sq2,chi2sq2_min,chi2sq2_max,maxhist_small);
				add_hist(bdif1,hbdif1_sess,bdif1_min,bdif1_max,maxhist);
				add_hist(bdif2,hbdif2_sess,bdif2_min,bdif2_max,maxhist);
								
				//timing
				add_hist(t1,ht1,t1_min,t1_max,maxhist_small);
				add_hist(t2,ht2,t2_min,t2_max,maxhist_small);
				add_hist(dt,hdt,dt_min,dt_max,maxhist_small);
				//cuts
				rf=bdif1_cal+bdif2_cal-2*511.0;
				db=bdif1_cal-bdif2_cal;
				//printf("%lf %lf %lf %lf\n",bdif1_cal,bdif2_cal,rf,db);
				if(fabs(db)<=rf_delta) add_hist(rf,hrf,hrf_min,hrf_max,maxhist_2D);
				if(fabs(rf)<=db_delta) add_hist(db,hdb,hdb_min,hdb_max,maxhist_2D);
				//2D hist
				add_2Dhist(db,rf,ee,hrf_min,hrf_max,maxhist_2D);
				//aif 
				if(fabs(rf-(sqrt(pow(db,2)+pow(511.0,2))-511.0))<d_aif) add_hist(db,haif,aif_min,aif_max,maxhist);

				fwrite(&bdif1_cal,sizeof(bdif1_cal),1,febin);
				fwrite(&bdif2_cal,sizeof(bdif2_cal),1,febin);
				fwrite(&dt,sizeof(dt),1,febin);

			}//i_good	
				
			}// if i_mode==3


			
 
		if(i_good1==1) n_good1++;
		if(i_good2==1) n_good2++;
		if(i_good1==1 && i_good2==1) n_good++;

		if(i_mode==0 || i_mode==1) 	printf("waveform: %d (det1: %6.2lf %% det2: %6.2lf %%) \r",j,(double)n_good1/(double)n_all*100.0,(double)n_good2/(double)n_all*100.0);
		if(i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) printf("waveform: %d (good: %6.2lf %% det1: %6.2lf %% det2: %6.2lf %%) \r",j,(double)n_good/(double)n_all*100.0,(double)n_good1/(double)n_all*100.0,(double)n_good2/(double)n_all*100.0);
			
		
	//
		
	}//j
	printf("\n");
	fclose(fsp);
	
	if(i_mode==0)
	{
	
		get_imax2(&rpom,&ch_bcg1_sess[l],&chi2_pom,hbcg1_mean_l_sess,0,maxhist_small-1,15);
		get_imax2(&rpom,&ch_bcg2_sess[l],&chi2_pom,hbcg2_mean_l_sess,0,maxhist_small-1,15);
		//get_imax(&ipom,&ch_bcg1_sess[l],hbcg1_mean_l_sess,0,maxhist_small-1);
		//get_imax(&ipom,&ch_bcg2_sess[l],hbcg2_mean_l_sess,0,maxhist_small-1);
		bcg1_sess[l]=bcg1_mean_l_min+ch_bcg1_sess[l]*(bcg1_mean_l_max-bcg1_mean_l_min)/maxhist_small;
		bcg2_sess[l]=bcg2_mean_l_min+ch_bcg2_sess[l]*(bcg1_mean_l_max-bcg2_mean_l_min)/maxhist_small;
		get_imax2(&rpom,&ch_bdif1_max[l],&chi2_pom,hbdif1_sess,0,maxhist-1,15);
		get_imax2(&rpom,&ch_bdif2_max[l],&chi2_pom,hbdif2_sess,0,maxhist-1,15);
		//get_imax(&ipom,&ch_bdif1_max[l],hbdif1_sess,0,maxhist-1);
		//get_imax(&ipom,&ch_bdif2_max[l],hbdif2_sess,0,maxhist-1);
		bdif1_sess[l]=bdif1_min+ch_bdif1_max[l]*(bdif1_max-bdif1_min)/maxhist;
		bdif2_sess[l]=bdif2_min+ch_bdif2_max[l]*(bdif2_max-bdif2_min)/maxhist;
		bdif1_sess_avg=bdif1_sess_avg+bdif1_sess[l]/(double)n_sess;
		bdif2_sess_avg=bdif2_sess_avg+bdif2_sess[l]/(double)n_sess;
				
		//calibration
		if(bdif1_peak1_min==0 && bdif1_peak1_max==0) bdif1_peak1=w1_peak1=0;
		else get_calib(&bdif1_peak1,&w1_peak1,bdif1_peak1_min,bdif1_peak1_max,bdif1_min,bdif1_max,hbdif1_sess);
		if(bdif2_peak1_min==0 && bdif2_peak1_max==0) bdif2_peak1=w2_peak1=0;
		else get_calib(&bdif2_peak1,&w2_peak1,bdif2_peak1_min,bdif2_peak1_max,bdif2_min,bdif2_max,hbdif2_sess);
		get_calib(&bdif1_peak2,&w1_peak2,bdif1_peak2_min,bdif1_peak2_max,bdif1_min,bdif1_max,hbdif1_sess);
		get_calib(&bdif2_peak2,&w2_peak2,bdif2_peak2_min,bdif2_peak2_max,bdif2_min,bdif2_max,hbdif2_sess);
		a1_cal=(cal_energy2-cal_energy1)/(bdif1_peak2-bdif1_peak1);
		a2_cal=(cal_energy2-cal_energy1)/(bdif2_peak2-bdif2_peak1);
		fprintf(fcal,"%6ld %18.6lf %18.6lf %18.6lf ",l,bcg1_sess[l],bdif1_peak1,w1_peak1*a1_cal); 
		fprintf(fcal,"%18.6lf %18.6lf ",bdif1_peak2,w1_peak2*a1_cal); 
		fprintf(fcal,"%18.6lf %18.6lf %18.6lf ",bcg2_sess[l],bdif2_peak1,w2_peak1*a2_cal); 
		fprintf(fcal,"%18.6lf %18.6lf \n",bdif2_peak2,w2_peak2*a2_cal); 
		
	} //if i_mode==0

	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5)
	{
	
		get_imax2(&rpom,&ch_bcg1_sess[l],&chi2_pom,hbcg1_mean_l_sess,0,maxhist_small-1,15);
		get_imax2(&rpom,&ch_bcg2_sess[l],&chi2_pom,hbcg2_mean_l_sess,0,maxhist_small-1,15);
		//get_imax(&ipom,&ch_bcg1_sess[l],hbcg1_mean_l_sess,0,maxhist_small-1);
		//get_imax(&ipom,&ch_bcg2_sess[l],hbcg2_mean_l_sess,0,maxhist_small-1);
		bcg1_sess[l]=bcg1_mean_l_min+ch_bcg1_sess[l]*(bcg1_mean_l_max-bcg1_mean_l_min)/maxhist_small;
		bcg2_sess[l]=bcg2_mean_l_min+ch_bcg2_sess[l]*(bcg1_mean_l_max-bcg2_mean_l_min)/maxhist_small;
		get_imax2(&rpom,&ch_bdif1_max[l],&chi2_pom,hbdif1_sess,0,maxhist-1,15);
		get_imax2(&rpom,&ch_bdif2_max[l],&chi2_pom,hbdif2_sess,0,maxhist-1,15);
		//get_imax(&ipom,&ch_bdif1_max[l],hbdif1_sess,0,maxhist-1);
		//get_imax(&ipom,&ch_bdif2_max[l],hbdif2_sess,0,maxhist-1);
		bdif1_sess[l]=bdif1_min+ch_bdif1_max[l]*(bdif1_max-bdif1_min)/maxhist;
		bdif2_sess[l]=bdif2_min+ch_bdif2_max[l]*(bdif2_max-bdif2_min)/maxhist;
		bdif1_sess_avg=bdif1_sess_avg+bdif1_sess[l]/(double)n_sess;
		bdif2_sess_avg=bdif2_sess_avg+bdif2_sess[l]/(double)n_sess;
		//printf("%lf %lf %lf %ld\n",bdif2_sess[l],bdif2_min,bdif2_max,ch_bdif2_max[l]);
		
		//calibration
		if(bdif1_peak1_min==0 && bdif1_peak1_max==0) bdif1_peak1=w1_peak1=0;
		else get_calib(&bdif1_peak1,&w1_peak1,bdif1_peak1_min,bdif1_peak1_max,bdif1_min,bdif1_max,hbdif1_sess);
		if(bdif2_peak1_min==0 && bdif2_peak1_max==0) bdif2_peak1=w2_peak1=0;
		else get_calib(&bdif2_peak1,&w2_peak1,bdif2_peak1_min,bdif2_peak1_max,bdif2_min,bdif2_max,hbdif2_sess);
		get_calib(&bdif1_peak2,&w1_peak2,bdif1_peak2_min,bdif1_peak2_max,bdif1_min,bdif1_max,hbdif1_sess);
		get_calib(&bdif2_peak2,&w2_peak2,bdif2_peak2_min,bdif2_peak2_max,bdif2_min,bdif2_max,hbdif2_sess);

						
		a1_cal=(cal_energy2-cal_energy1)/(bdif1_peak2-bdif1_peak1);
		a2_cal=(cal_energy2-cal_energy1)/(bdif2_peak2-bdif2_peak1);
		fprintf(fcal1,"%6ld %18.6lf %18.6lf %18.6lf ",l,bcg1_sess[l],bdif1_peak1,w1_peak1*a1_cal); 
		fprintf(fcal1,"%18.6lf %18.6lf ",bdif1_peak2,w1_peak2*a1_cal); 
		fprintf(fcal1,"%18.6lf %18.6lf %18.6lf ",bcg2_sess[l],bdif2_peak1,w2_peak1*a2_cal); 
		fprintf(fcal1,"%18.6lf %18.6lf \n",bdif2_peak2,w2_peak2*a2_cal); 
			
	} //if i_mode==1
	
	}//l main loop end
	if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) fclose(fcal1);
	
	if(i_mode==-1)
	{
		fclose(fwav1);
		fclose(fwav2);
		fclose(fwdif1);
		fclose(fwdif2);
		fclose(fparabola1);
		fclose(fparabola2);
	}
	
	if(i_mode==2 || i_mode==3 ||i_mode==4 || i_mode==5) fclose(febin);
	
		
		if(i_mode==0 || i_mode==1 || i_mode==2 || i_mode==4)
		{
			fpeakm=fopen("peakm.txt","w");
			
			for(i=0;i<maxhist_px;i++)
			{
			
				get_imax2(&peak1_max,&ch_peak1m,&chi2_peak1m,pp1[i],0,maxhist_py-1,5);
				peak1m[i]=peak1m_min+ch_peak1m*(peak1m_max-peak1m_min)/(double)maxhist_py;
				get_wl(&wpeak1_l,pp1[i],peak1_max,(int)ch_peak1m);
				peak1l[i]=peak1m_min+(ch_peak1m-wpeak1_l)*(peak1m_max-peak1m_min)/(double)maxhist_py;
				get_wr(&wpeak1_r,pp1[i],peak1_max,(int)ch_peak1m);
				peak1r[i]=peak1m_min+(ch_peak1m+wpeak1_r)*(peak1m_max-peak1m_min)/(double)maxhist_py;
				get_imax2(&peak2_max,&ch_peak2m,&chi2_peak2m,pp2[i],0,maxhist_py-1,5);
				peak2m[i]=peak2m_min+ch_peak2m*(peak2m_max-peak2m_min)/(double)maxhist_py;
				get_wl(&wpeak2_l,pp2[i],peak2_max,(int)ch_peak2m);
				peak2l[i]=peak2m_min+(ch_peak2m-wpeak2_l)*(peak2m_max-peak2m_min)/(double)maxhist_py;
				get_wr(&wpeak2_r,pp2[i],peak2_max,(int)ch_peak2m);
				peak2r[i]=peak2m_min+(ch_peak2m+wpeak2_r)*(peak2m_max-peak2m_min)/(double)maxhist_py;
			}
			
					
			for(i=0;i<maxhist_px;i++)
			{
				fprintf(fpeakm,"%6ld %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf\n",i,peak1m[i],peak1l[i],peak1r[i],peak2m[i],peak2l[i],peak2r[i]);
			}
			fclose(fpeakm);
			
			fpeakh=fopen("fpeakh.txt","w");
			for(i=0;i<maxhist_py;i++)	fprintf(fpeakh,"%6ld %8ld %8ld %8ld\n",i,pp1[400][i],pp1[500][i],pp1[600][i]);
			fclose(fpeakh);
			
			fstab=fopen("stability.txt","w");
			for(l=1;l<=n_sess;l++) 
			{
				fprintf(fstab,"%6ld %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf\n",l,bcg1_sess[l],bdif1_sess[l],1+(bdif1_sess[l]-bdif1_sess_avg)/bdif1_sess_avg,bcg2_sess[l],bdif2_sess[l],1+(bdif2_sess[l]-bdif2_sess_avg)/bdif2_sess_avg);
			
			}
			fclose(fstab);
		}// i_mode==0
		
		 //zapis
		printf("writing files ...\n");
		w_hist(hdarea1,hdarea2,"hist-darea.txt",darea1_min,darea1_max,darea2_min,darea2_max,maxhist_small);
		w_hist(hwdif1_max,hwdif2_max,"hist-wdif_max.txt",wdif1_max_min,wdif1_max_max,wdif2_max_min,wdif2_max_max,maxhist_small);
				
		w_hist(hwmax1,hwmax2,"hist-wmax.txt",hwmax1_min,hwmax1_max,hwmax2_min,hwmax2_max,maxhist);
		w_hist(hch_wmax1,hch_wmax2,"hist-ch_wmax.txt",hch_wmax1_min,hch_wmax1_max,hch_wmax2_min,hch_wmax2_max,maxhist_small2);
		w_hist(hch_darea1,hch_darea2,"hist-ch_darea.txt",hch_darea1_min,hch_darea1_max,hch_darea2_min,hch_darea2_max,maxhist_small2);

		w_hist(hbcg1_mean_l,hbcg2_mean_l,"hist-bcg_mean_l.txt",bcg1_mean_l_min,bcg1_mean_l_max,bcg2_mean_l_min,bcg2_mean_l_max,maxhist_small);
		w_hist(hbcg1_sigma_l,hbcg2_sigma_l,"hist-bcg_sigma_l.txt",bcg1_sigma_l_min,bcg1_sigma_l_max,bcg2_sigma_l_min,bcg2_sigma_l_max,maxhist_small);
		w_hist(hchi2l_1,hchi2l_2,"hist-chi2l.txt",chi2l_1_min,chi2l_1_max,chi2l_2_min,chi2l_2_max,maxhist_small);
				
		w_hist(hbdif1,hbdif2,"hist-bdif.txt",bdif1_min,bdif1_max,bdif2_min,bdif2_max,maxhist);
		w_hist(hchi2sq1,hchi2sq2,"hist-chi2sq.txt",chi2sq1_min,chi2sq1_max,chi2sq2_min,chi2sq2_max,maxhist_small);
		w_hist(hbdif1_sess,hbdif2_sess,"hist-bdif-sess.txt",bdif1_min,bdif1_max,bdif2_min,bdif2_max,maxhist);
		
		if(i_mode==1 || i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
		{
			w_hist(hbdif1_cal,hbdif2_cal,"hist-bdif-cal.txt",bdif1_cal_min,bdif1_cal_max,bdif2_cal_min,bdif2_cal_max,maxhist);
		}
		
		if(i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5)
		{
			w_hist(ht1,ht2,"hist-t.txt",t1_min,t1_max,t2_min,t2_max,maxhist_small);
			w_hist(hdt,hdt,"hist-dt.txt",dt_min,dt_max,dt_min,dt_max,maxhist_small);
			w_hist(hrf,hdb,"cuts.txt",hrf_min,hrf_max,hdb_min,hdb_max,maxhist_2D);
			w_2Dhist(ee,"ee.txt",hrf_min,hrf_max,maxhist_2D);
			get_2Dmax(&ch_max_db,&ch_max_rf,ee);
			//w_2Dcuts(ch_max_db,ch_max_rf,ee,hdb_min,hdb_max,hrf_min,hrf_max);
			w_hist(haif,haif,"hist-aif.txt",aif_min,aif_max,aif_min,aif_max,maxhist);
		}
		
		
		//if(i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) fclose(fcal1);
		
		//avegared cal for cdb
		if(i_mode==2 || i_mode==3 || i_mode==4 || i_mode==5) 
		{
			fcal1=fopen("e1-cal-cdb.avg","w");
			if(bdif1_peak1_min==0 && bdif1_peak1_max==0) bdif1_peak1=w1_peak1=0;
			else get_calib(&bdif1_peak1,&w1_peak1,bdif1_peak1_min,bdif1_peak1_max,bdif1_min,bdif1_max,hbdif1);
			if(bdif2_peak1_min==0 && bdif2_peak1_max==0) bdif2_peak1=w2_peak1=0;
			else get_calib(&bdif2_peak1,&w2_peak1,bdif2_peak1_min,bdif2_peak1_max,bdif2_min,bdif2_max,hbdif2);
			get_calib(&bdif1_peak2,&w1_peak2,bdif1_peak2_min,bdif1_peak2_max,bdif1_min,bdif1_max,hbdif1);
			get_calib(&bdif2_peak2,&w2_peak2,bdif2_peak2_min,bdif2_peak2_max,bdif2_min,bdif2_max,hbdif2);
			w1_peak1=w1_peak1*(cal_energy2-cal_energy1)/(bdif1_peak2-bdif1_peak1);
			w2_peak1=w2_peak1*(cal_energy2-cal_energy1)/(bdif2_peak2-bdif2_peak1);
			w1_peak2=w1_peak2*(cal_energy2-cal_energy1)/(bdif1_peak2-bdif1_peak1);
			w2_peak2=w2_peak2*(cal_energy2-cal_energy1)/(bdif2_peak2-bdif2_peak1);
			fprintf(fcal1,"%18.6lf %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf %18.6lf\n",bdif1_peak1,w1_peak1,bdif1_peak2,w1_peak2,bdif2_peak1,w2_peak1,bdif2_peak2,w2_peak2);
			fclose(fcal1);
		}
		
		fpeak_mean=fopen("peak_mean.txt","w");
		for(i=0;i<n_points;i++) 
		{
			pp1_mean[i]=pp1_mean[i]/(double)pp1_counter;
			pp2_mean[i]=pp2_mean[i]/(double)pp2_counter;
			fprintf(fpeak_mean,"%6ld %18.6lf %18.6lf\n",i,pp1_mean[i],pp2_mean[i]); 
		}
		fclose(fpeak_mean);
		
		fstat=fopen("statistics.txt","w");
		fprintf(fstat,"%18.6lf %% good fraction det1\n",(double)n_good1/(double)n_all*100.0);
		fprintf(fstat,"%18.6lf %% good fraction det2\n",(double)n_good2/(double)n_all*100.0);
		fprintf(fstat,"%18.6lf %% good fraction det1+2\n",(double)n_good/(double)n_all*100.0);
		if(i_mode==1) fprintf(fstat,"2D max  DB: %ld, RF: %ld\n",ch_max_db,ch_max_rf); 
		fclose(fstat);

		fclose(flog);
		if(i_mode==0) 
		{
			free(pp1);
			free(pp2);
		}
		if(i_mode==2) free(ee);
		
		
		
}
