/*
 *  Copyright (C) 2012  Marian Vlcek, Jakub Cizek
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *  If you publish any work which uses this program and/or data
 *  produced by this program, please cite following paper: 
 * 
 *  J. Cizek. M. Vlcek, I. Prochazka, Digital spectrometer
 *  for coincidence measurement of Doppler broadening of positron
 *  annihilation radiation, Nuclear Instruments and Methods
 *  in Physics Research, Section A 623, 982-994 (2010)
 */

#ifndef HISTOGRAM_H
#define HISTOGRAM_H

#include "boost/multi_array.hpp"

struct Window {
public:
	int len;
	double *win;
	Window(double (*func)(int,int), int l);
};

class histogram1D
{
	private:
		int bins;
		int channels;
		double lower;
		double upper;
		typedef boost::multi_array<double, 2> array_type;
	public:
		array_type hist;
		histogram1D(const int b, const int c, const double l, const double u);
		~histogram1D();
		void add(const double val, const int c);
		void write(const char path[]);
		void calibrate(const double a, const double b);
		void smooth(const Window window);
		double fwhm(const double x, const int c);
		void clearData();
};

class histogram2D
{
	private:
		int x_bins;
		int y_bins;		
		int channels;
		double x_lower;
		double x_upper;
		double y_lower;
		double y_upper;		
		typedef boost::multi_array<double, 3> array_type;
	public:
		array_type hist;
		histogram2D(const int x_b, const int y_b, const int c, const double x_l, const double x_u, const double y_l, const double y_u);
		~histogram2D();
		void add(const double x_val, const double y_val, const int c);
		void writeText(const char path[]);
		void writeImage(const char path[], const bool logscale);
		void writeConstXCut(const char path[], const int x, const bool normalize);
		void writeConstYCut(const char path[], const int y, const bool normalize);
		void writeConstXCutWide(const char path[], const int x, const int w, const bool normalize);
		void writeConstYCutWide(const char path[], const int y, const int w, const bool normalize);
		double fwhmConstXCutWide(const int x, const double val, const int w, const int c);
};

struct color {
	unsigned char r, g, b;
};

void interpolate(double val, int *r, int *g, int *b, struct color a[], const int n);

#endif // HISTOGRAM_H
